@@ -25,11 +25,12 @@ SOFTWARE.
 
 #include <X11/Xlibint.h>
 #include "FrameMgr.h"
+#include <glib.h>
 
 /* Convenient macro */
 
-#define _UNIT(n)   ((int)(n) & 0xff)
-#define _NUMBER(n) (((int)(n) >> 8) & 0xff)
+#define _UNIT(n)   (GPOINTER_TO_INT(n) & 0xff)
+#define _NUMBER(n) ((GPOINTER_TO_INT(n) >> 8) & 0xff)
 
 /* For byte swapping */
 
@@ -188,7 +189,7 @@ FrameMgrInit (XimFrame frame, char* area, Bool byte_swap)
 {
     FrameMgr fm;
 
-    fm = (FrameMgr)Xmalloc(10000*sizeof(FrameMgrRec));
+    fm = (FrameMgr)Xmalloc(sizeof(FrameMgrRec));
 
     fm->frame = frame;
     fm->fi = FrameInstInit(frame);
@@ -540,8 +541,7 @@ FrameMgrSkipToken(FrameMgr fm, int skip_count)
 	    fm->idx += 2;
 	} else if (type == BIT32) {
 	    fm->idx += 4;
-	}
-	  else if (type == BARRAY && info.num != NO_VALUE) {
+	} else if (type == BARRAY && info.num != NO_VALUE) {
 	    fm->idx += info.num;
 	} else if (type == BARRAY && info.num == NO_VALUE) {
 	    return FmInvalidCall;
@@ -678,8 +678,8 @@ FrameInstGetNextType(FrameInst fi, XimFrameTypeInfo info)
 	    register int offset, iter_idx;
 
 	    info->counter.is_byte_len = 
-		((int)fi->template[fi->cur_no].data & 0xff) == FmCounterByte;
-	    offset = (int)fi->template[fi->cur_no].data >> 8;
+		(GPOINTER_TO_INT(fi->template[fi->cur_no].data) & 0xff) == FmCounterByte;
+	    offset = GPOINTER_TO_INT(fi->template[fi->cur_no].data) >> 8;
 	    iter_idx = fi->cur_no + offset;
 	    if (fi->template[iter_idx].type == ITER) {
 		ExtraData d;
@@ -798,8 +798,8 @@ FrameInstPeekNextType(FrameInst fi, XimFrameTypeInfo info)
 	    register int offset, iter_idx;
 
 	    info->counter.is_byte_len = 
-		((int)fi->template[fi->cur_no].data & 0xff) == FmCounterByte;
-	    offset = (int)fi->template[fi->cur_no].data >> 8;
+		(GPOINTER_TO_INT(fi->template[fi->cur_no].data) & 0xff) == FmCounterByte;
+	    offset = GPOINTER_TO_INT(fi->template[fi->cur_no].data) >> 8;
 	    iter_idx = fi->cur_no + offset;
 	    if (fi->template[iter_idx].type == ITER) {
 		ExtraData d;
