@@ -33,6 +33,9 @@ IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "i18nUtil.h"
 #define IC_SIZE 64
 
+#define Swap32(n)  (((n) << 24 & 0xff000000) | ((n) <<  8 & 0xff0000) |   \
+                    ((n) >>  8 & 0xff00)     | ((n) >> 24 & 0xff))
+
 extern void do_gc(char *);
 
 /* Set IC values */
@@ -133,9 +136,6 @@ SetPointAttribute(XICAttribute *value_ret, char *p, XICAttr *ic_attr,
     /* free FrameMgr */
     FrameMgrFree(fm);
 
-    memmove(&(buf->x), p, sizeof(INT16)); p += sizeof(INT16);
-    memmove(&(buf->y), p, sizeof(INT16));
-
     value_ret->attribute_id = ic_attr->attribute_id;
     value_ret->name = ic_attr->name;
     value_ret->name_length = ic_attr->length;
@@ -776,7 +776,10 @@ _Xi18nGetIC(XIMS ims, IMProtocol *call_data, unsigned char *p)
 				  _Xi18nNeedSwap(i18n_core, connect_id));
     if (status_ret != NULL)
       iter_count++;
-
+    if (ic_attr->value != NULL && _Xi18nNeedSwap(i18n_core, connect_id)) {
+	    *(CARD32 *)ic_attr->value = Swap32(*(CARD32 *)ic_attr->value);
+    }
+    
     /* create FrameMgr */
     fm = FrameMgrInit(get_ic_values_reply_fr, NULL,
 		      _Xi18nNeedSwap(i18n_core, connect_id));
