@@ -406,7 +406,7 @@ SetXi18nSelectionOwner(Xi18n i18n_core)
     Atom realtype;
     int realformat;
     unsigned long bytesafter;
-    long *data;
+    unsigned char *data;
     unsigned long length;
     Atom atom;
     int i;
@@ -438,7 +438,7 @@ SetXi18nSelectionOwner(Xi18n i18n_core)
     }
     found = False;
     for (i = 0; i < length; i++) {
-	if (data[i] == atom) {
+	if (((Atom *)data)[i] == atom) {
 	    Window owner;
 	    found = True;
 	    if ((owner = XGetSelectionOwner(dpy, atom)) != ims_win) {
