@@ -39,6 +39,7 @@ IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "i18nUtil.h"
 #include "i18nIc.h"
 
+#define Swap16(n) (((n) << 8 & 0xff00) | ((n) >> 8 & 0xff))
 
 static void
 GetProtocolVersion(CARD16 client_major, CARD16 client_minor,
@@ -855,8 +856,11 @@ ResetICMessageProc(XIMS ims, IMProtocol *call_data, unsigned char *p)
 
 static int
 WireEventToEvent(Xi18n i18n_core, xEvent *event,
-		 CARD16 serial, XEvent *ev)
+		 CARD16 serial, XEvent *ev, Bool byte_swap)
 {
+    if (byte_swap != 0) {
+        event->u.keyButtonPointer.state = Swap16(event->u.keyButtonPointer.state);
+    }
     ev->xany.serial = event->u.u.sequenceNumber & ((unsigned long) 0xffff);
     ev->xany.serial |= serial << 16;
     ev->xany.send_event = False;
@@ -908,7 +912,8 @@ ForwardEventMessageProc(XIMS ims, IMProtocol *call_data, unsigned char *p)
     FrameMgrFree(fm);
 
     if (WireEventToEvent(i18n_core, &wire_event,
-			 forward->serial_number,&forward->event) == True) {
+			 forward->serial_number,&forward->event,
+			 _Xi18nNeedSwap (i18n_core, connect_id)) == True) {
 	if (i18n_core->address.improto)
 	  if (!(i18n_core->address.improto(ims, call_data)))
 	    return;
