@@ -12,9 +12,10 @@ if [ x$USE_LIBTOOL = xyes ]
 then
   libtoolize --force --automake
 fi
-aclocal $ACLOCAL_FLAGS
+gettextize -c -f --intl --no-changelog
+aclocal-1.4 $ACLOCAL_FLAGS
 autoheader
-automake --add-missing $STRICTNESS
+automake-1.4 --add-missing --copy $STRICTNESS
 autoconf
 
 for i in $SUBPACKAGES
