@@ -1,4 +1,4 @@
-/* this file is a part of Ami software, (C) Hwang chi-deok 1999 */
+/* This file is a part of Ami software, (C) Hwang chi-deok 1999, 2001 */
 
 #include "config.h"
 
@@ -11,6 +11,7 @@
 #include <Ximd/Xi18n.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <langinfo.h>
 
 #include "ami.h"
 #include "applet.h"
@@ -19,6 +20,7 @@
 #include "util.h"
 #include "keyparse.h"
 #include "conf.h"
+#include "comp.h"
 
 static void print_help(void);
 
@@ -31,16 +33,20 @@ long filter_mask = KeyPressMask;
 GdkFont	*default_xim_font = NULL;
 GdkBitmap *dim_mask;
 int dock_style = WM_DETECT;
-gboolean allow_shuffle = FALSE;
 
-XIMS	xims;
+XIMS xims;
 char *hangul_mode_label = NULL;
 char *english_mode_label = NULL;
 
+int ami_codeset;
+
 #define DEFAULT_IMNAME "Ami"
 #define DEFAULT_LOCALE "ko"
-#define DEFAULT_FONT1	"lucidasans-14,-*-*-medium-r-normal--*-*-*-*-c-160-ksc5601.1987-0"
-#define DEFAULT_FONT2	"*,-*-*-medium-r-normal--16-*-*-*-c-*-*"
+#define DEFAULT_FONT1_EUC      "lucidasans-14,-*-*-medium-r-normal--*-*-*-*-c-160-ksc5601.1987-0"
+#define DEFAULT_FONT2_EUC      "*,-*-*-medium-r-normal--16-*-*-*-c-*-*"
+#define DEFAULT_FONT1_UTF8     "-*-*-medium-r-normal-ko-16-*-*-*-c-*-iso10646-1,-*-*-medium-r-normal--16-*-*-*-c-*-*-*"
+#define DEFAULT_FONT2_UTF8     "-*-*-medium-r-normal-ko-18-*-*-*-c-*-iso10646-1,-*-*-medium-r-normal--18-*-*-*-*-*-iso10646-1" 
+
 
 char *imname = DEFAULT_IMNAME;
 char *default_fontname = NULL;
@@ -50,23 +56,26 @@ GtkWidget *cp_win = NULL, *cp = NULL;
 
 gint ami_run_mode = AMI_NORMAL_RUN;
 int ami_editing_mode = AMI_EDITING_WORD_MODE;
-int ami_keyboard = 0;
-int ami_ekeyboard = 1;
 int bell_intensity = 0;
 
+int num_fonts_in_utf8_fs = 7;
+
 gboolean ami_line_wraping_mode = FALSE;
 gboolean ami_use_underline = TRUE;
 gboolean esc_han_toggle = FALSE;
 gboolean unique_han = FALSE;
 gboolean support_status = TRUE;
 gboolean support_cb = TRUE;
-gboolean wait_wm = FALSE; 
+gboolean wait_wm = FALSE;
+gboolean allow_shuffle = FALSE;
 
 int ami_hanja_def_subst_mode = 1;
 int ami_hanja_subst_mode = 0;
 char * ami_hanja_subst_format = NULL;
 
 char *pix_dir_name = NULL;
+char *keyboard_name = NULL;
+
 
 /* Supported Inputstyles */
 static XIMStyle Styles[] = {
@@ -112,10 +121,10 @@ static XIMStyle Styles_nostatus[] = {
     0
 };
 
-/* IM   Ʈ Ű  ü */
+/* struct for trigger keys used to open IM */
 XIMTriggerKeys *on_keys = NULL;
 
-/* ַ handler_match_keys[2] Լ ̵ ü */
+/* structs mainly used in hanlder_match_keys[2] functions */
 XIMTriggerKey *trigger_keys = NULL;
 XIMTriggerKey *hanja_keys = NULL;
 XIMTriggerKey *special_char_keys = NULL;
@@ -127,6 +136,7 @@ static XIMEncoding koEncodings[] = {
     NULL
 };
 
+static char *locale=NULL;
 
 int
 proto_handler(XIMS ims, IMProtocol *call_data)
@@ -192,19 +202,17 @@ ami_init_im(GtkWidget* win)
     }
     if (!support_cb) {
 	if (debug) printf("callback is disabled\n");
-	input_styles->count_styles = sizeof(Styles_nocb)/sizeof(XIMStyle) - 1;
+	input_styles->count_styles = sizeof(Styles_nocb)/sizeof(Styles_nocb[0]) - 1;
 	input_styles->supported_styles = Styles_nocb;
     } else if (unique_han || !support_status) {
-	input_styles->count_styles = sizeof(Styles_nostatus)/sizeof(XIMStyle) - 1;
+	if (debug) printf("no status area\n");
+	input_styles->count_styles = sizeof(Styles_nostatus)/sizeof(Styles_nostatus[0]) - 1;
 	input_styles->supported_styles = Styles_nostatus;
     } else {
-	input_styles->count_styles = sizeof(Styles)/sizeof(XIMStyle) - 1;
+	input_styles->count_styles = sizeof(Styles)/sizeof(Styles[0]) - 1;
 	input_styles->supported_styles = Styles;
     }
 
-
-
-
     if ((encodings = (XIMEncodings *)malloc(sizeof(XIMEncodings))) == NULL) {
 	fprintf(stderr, "Can't allocate\n");
 	exit(1);
@@ -222,9 +230,9 @@ ami_init_im(GtkWidget* win)
 		   IMInputStyles, input_styles,
 		   NULL);
     if (xims == (XIMS)NULL) {
-	g_warning("Can't Open Input Method Service:\n"
-	         "\tInput Method Name :%s\n" , imname);
-	g_print("***Please check whether another ami is running\n\n");
+	g_warning(_("Can't Open Input Method Service:\n"
+	         "\tInput Method Name :%s\n") , imname);
+	g_print(_("***Please check whether another ami is running\n\n"));
 	exit(1);
     }
     if (use_trigger) {
@@ -253,74 +261,60 @@ ami_init_im(GtkWidget* win)
 	char dim_data[] = {0x02, 0x01};
 	dim_mask = gdk_bitmap_create_from_data(NULL, dim_data, 2, 2);
     }
-    if (default_fontname)
+    if (default_fontname && *default_fontname)
 	default_xim_font = util_fontset_load(default_fontname);
-    if (!default_xim_font) default_xim_font = util_fontset_load(DEFAULT_FONT1);
-    if (!default_xim_font) default_xim_font = util_fontset_load(DEFAULT_FONT2);
+    if (ami_codeset == AMI_EUC ) 
+    {
+  	if (!default_xim_font)
+	    default_xim_font = util_fontset_load(DEFAULT_FONT1_EUC);
+    	if (!default_xim_font)
+	    default_xim_font = util_fontset_load(DEFAULT_FONT2_EUC);
+    }
+    else
+    {
+    	if (!default_xim_font)
+	    default_xim_font = util_fontset_load(DEFAULT_FONT1_UTF8);
+    	if (!default_xim_font)
+	    default_xim_font = util_fontset_load(DEFAULT_FONT2_UTF8);
+    }
+
+    if (!default_xim_font)
+	g_error("Cannot set XIM default font\n");
 
     return 0;
 }
 
-#if 0
-static gint
-show_main_win(gpointer data)
-{
-    gtk_widget_show_all(cp_win);
-    return FALSE;
-}
-#endif
-
-int 
+int
 ami_main(int argc, char *argv[], char *rc_file)
 {
     XWMHints *wm_hints;
-    
+
     gtk_init(&argc, &argv);
     gtk_rc_parse(rc_file);
 
     cp_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title(GTK_WINDOW(cp_win), "ƹ ");
+    gtk_window_set_title(GTK_WINDOW(cp_win), _("Ami Preference"));
 
-    /******/
     gtk_widget_realize(cp_win);
     wm_hints = XAllocWMHints();
     wm_hints->flags = StateHint;
     wm_hints->initial_state = IconicState;
     XSetWMHints(gdk_display, GDK_WINDOW_XWINDOW(cp_win->window), wm_hints);
     XFree(wm_hints);
-    /******/
-#if 0
-    {
-#define WinStateAllWorkspaces (1 << 0)
-#define WinStateMinimized (1 << 1)
-        XClientMessageEvent xev;
-	xev.type = ClientMessage;
-	xev.window = GDK_WINDOW_XWINDOW(cp_win->window);
-	xev.message_type = XInternAtom(GDK_DISPLAY(), "_WIN_STATE", False);
-	xev.format = 32;
-	xev.data.l[0] = WinStateAllWorkspaces | WinStateMinimized;
-	xev.data.l[1] = WinStateAllWorkspaces | WinStateMinimized;
-	xev.data.l[2] = gdk_time_get();
 
-	XSendEvent(GDK_DISPLAY(), GDK_ROOT_WINDOW(), False,
-		  SubstructureNotifyMask, (XEvent *) & xev);
-    }
-#endif /* 0 */
-    
     gtk_container_set_border_width(GTK_CONTAINER(cp_win), 6);
     gtk_signal_connect(GTK_OBJECT(cp_win), "destroy", (GtkSignalFunc)cp_quit, NULL);
 
     cp = create_cp();
     gtk_container_add(GTK_CONTAINER(cp_win), cp);
 
-    //gtk_timeout_add(2000, show_main_win, NULL);
     gtk_widget_show_all(cp_win);
 
     ami_init_im(cp_win);
 
     XBell(gdk_display, bell_intensity);
     gtk_main();
-    
+
     return 0;
 }
 
@@ -330,22 +324,70 @@ ami_exit_func(void)
     if (xims) IMCloseIM(xims);
 }
 
+#ifdef DEBUG_WITH_GDB
+#include <signal.h>
+static void
+caughtSignal(int sig)
+{
+#define ____NAME "ami"
+#define ____PATH "/usr/bin/ami"
+#define ____TEMP "/tmp/.pcgdb.XXXXXX"
+
+    fprintf(stderr, "%s: caught signal %d\n", ____NAME, sig);
+
+    if (1)
+    {
+	char tmpFileName[] = ____TEMP;
+	char cmd[256];
+	FILE *fp;
+
+	mkstemp(tmpFileName);
+	if ((fp = fopen(tmpFileName, "w")) != NULL)
+	{
+	    fprintf(fp,
+		    "file %s\n"
+		    "attach %d\n"
+		    "set height 1000\n"
+		    "bt full\n"
+		    , ____PATH, getpid());
+	    fclose(fp);
+	    snprintf(cmd, sizeof(cmd),
+		     "hanterm -sb -sl 1024 -e gdb -x %s", tmpFileName);
+	    system(cmd);
+	    unlink(tmpFileName);
+	}
+    }
+
+    exit(1);
+}
+#endif
+
 int
 main(int argc, char *argv[])
 {
     char *rc_file;
-    char *locale;
     gint i;
+    char *codeset;
+
+#ifdef HAVE_GETTEXT
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
+#endif
+
+#ifdef DEBUG_WITH_GDB
+    signal(SIGSEGV, caughtSignal);
+#endif
 
     ami_config_init();
 
-    if (strstr(argv[0], "wmami")) ami_run_mode = AMI_WM_RUN; 
+    if (strstr(argv[0], "wmami")) ami_run_mode = AMI_WM_RUN;
 #ifdef GNOME_APPLET
-    if (strstr(argv[0], "ami_applet")) ami_run_mode = AMI_GNOME_RUN; 
+    if (strstr(argv[0], "ami_applet")) ami_run_mode = AMI_GNOME_RUN;
 #endif
     for (i = 1; i < argc; i++) {
 	if (!strcmp(argv[i], "-wm")) {
-	    ami_run_mode = AMI_WM_RUN; 
+	    ami_run_mode = AMI_WM_RUN;
 	} else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
 	    print_help();
 	} else if (!strcmp(argv[i], "-debug")) {
@@ -368,7 +410,7 @@ main(int argc, char *argv[])
 	}
 #ifdef GNOME_APPLET
 	else if (!strcmp(argv[i], "-gnome")) {
-	    ami_run_mode = AMI_GNOME_RUN; 
+	    ami_run_mode = AMI_GNOME_RUN;
 	}
 #endif
     }
@@ -384,30 +426,43 @@ main(int argc, char *argv[])
 	    while((size = read(orig, buf, 256)) > 0) {
 		write(out, buf, size);
 	    }
-	    close(orig); close(out);
+	    close(orig);
+	    close(out);
 	}
     }
+
     locale = gtk_set_locale();
     if (!strcmp(locale, "C")) {
 	char *lang = g_getenv("LC_ALL");
 	if (!lang) lang = g_getenv("LC_CTYPE");
 	if (!lang) lang = g_getenv("LANG");
 	if (!lang) {
-	    g_print("LANG ȯ    Ǿʽϴ.");
-	    g_print("LANG  ko_KR.eucKR ѱ ȯ濡 ° Ͻ\n");
-	    g_print("ƹ̸ ٽ ϼ\n");
-	}
-	else if (strcmp(lang, "C") == 0 || strcmp(lang, "posix") == 0) {
-	    g_print("LANG ȯ溯 C  Ǿֽϴ.\n");
-	    g_print("LANG  koͰ ѱ ȯ濡 ° Ͻ \n");
-	    g_print("ƹ̸ ٽ Ͻʽÿ.\n");
+	    g_print(_("The environment variable LANG is not properly specified.\n"
+		      "Set it to one of Korean locales such as ko_KR.eucKR and\n"
+		      "ko_KR.UTF-8 and relaunch Ami\n"));
+	} else if (strcmp(lang, "C") == 0 || strcmp(lang, "posix") == 0) {
+	    g_print(_("The environment variable LANG is set to C.\n"
+		      "Set it to one of Korean locales such as ko_KR.eucKR\n"
+		      "and ko_KR.UTF-8 and relaunch Ami\n"));
 	} else if (strstr(lang, "ko")) {
-	    g_print(" locale ѱ ʴ \n");
-	    g_print("Դϴ. ý ü ġ ٽ غʽÿ\n");
+	    g_print(_("The current locale does not appear to  support Korean.\n"
+		      "Check your system configuration.\n"));
 	}
 	exit(1);
     }
 
+    codeset=nl_langinfo(CODESET);
+    if ( ! strcasecmp(codeset,"EUC-KR") ) 
+       ami_codeset=AMI_EUC;
+    else if ( ! strcasecmp(codeset,"UTF-8") )
+       ami_codeset=AMI_UTF8;
+    else {
+       g_print(_("The locale codeset is %s. It must be either "
+               "EUC-KR or UTF-8.\n Check your locale.\n"),
+               codeset);
+       exit(1); 
+    }
+
     g_atexit (ami_exit_func);
     switch(ami_run_mode) {
 	case AMI_WM_RUN:
@@ -420,7 +475,7 @@ main(int argc, char *argv[])
 #endif
 	case AMI_NORMAL_RUN:
 	default:
-	    ami_main(argc, argv, rc_file); 
+	    ami_main(argc, argv, rc_file);
     }
 
     exit(0);
@@ -429,10 +484,11 @@ main(int argc, char *argv[])
 static void
 print_help(void)
 {
-    g_print("ѱԷ± ƹ %s\n", VERSION);
-    g_print("ami -wm Ǵ wmami: WindowMaker E docking mode\n");
+    g_print(_("Korean Input Server Ami %s\n"), VERSION);
+    g_print(_("ami -wm or wmami: run Ami in the docking mode of Windowmaker or E.\n"));
 #ifdef GNOME_APPLET
-    g_print("ami -gnome Ǵ ami_applet: ׳ г ø \n");
+    g_print(_("ami -gnome or ami_applet: run Ami as a gnome panel applet.\n"));
 #endif
     exit(0);
 }
+
