@@ -5,21 +5,21 @@
 #include <gdk/gdkx.h>
 #include <Ximd/IMdkit.h>
 #include <Ximd/Xi18n.h>
+#include "comp.h"
 
 typedef struct _IC IC;
 typedef struct _HangulState HangulState;
 typedef struct _DispWindow DispWindow;
 
+#define HAN_STACK_SIZE 30
 struct _HangulState {
     unsigned char	*buf;
     int			buf_size;
     int			pos;
     int			len;
-    unsigned char	stack[20];
-    int			f, m, l;
-    int			sp, last_l, last_ll;
+    unsigned char	stack[HAN_STACK_SIZE];
+    Composer		*composer;
     int			composing_hangul;
-    int			has_temp_hangul;
     GtkWidget	*aux_input;
 };
 
@@ -36,8 +36,10 @@ struct _DispWindow {
     Colormap	cmap;		/* colormap */
     Pixmap	bg_pixmap;	/* background pixmap */
     int x, y;
-    int caret;  /* ѱ ۳ Ŀ ġ.  ѱ .
-                   minus  Է ǹ̰  status area Ÿ */
+    int caret;  /* the cursor position in terms of 'Hangul' character
+		   (syllable) relative to the beginning of hangul 
+		   buffer. A negative value indicates 'the cursor' is in
+		   the status area where input is meaningless. */
     int caret_pixel;
     int has_temp_hangul;
     int win_width;
@@ -69,17 +71,10 @@ enum {
     HOTKEY_CONTINUE
 };
 
-#define AUTOMATA_NEW 0
-#define AUTOMATA_CORRECT 1
-#define AUTOMATA_SPECIAL 2
-#define AUTOMATA_CORRECT_NEW 3
-#define AUTOMATA_ERROR 4
-#define AUTOMATA_NULL 5
-
 enum {
-    AMI_NORMAL_RUN,  /* Ϲ  */
-    AMI_WM_RUN,      /* WindowMaker Dock applet  */
-    AMI_GNOME_RUN    /* GNOME panle applet  */
+    AMI_NORMAL_RUN,  /* run as a standalone program */
+    AMI_WM_RUN,      /* run as a WindowMaker Dock applet */
+    AMI_GNOME_RUN    /* run as a GNOME panle applet */
 } AMI_RUN_MODE;
 
 enum { WM_E, WM_KDE1, WM_KDE2, WM_W, WM_DETECT, WM_UNKNOWN};
@@ -96,4 +91,9 @@ enum {
     AMI_PIXMAP_NONE
 } AMI_DRAWING_PIXMAP;
 
+enum {
+		AMI_EUC,
+		AMI_UTF8
+} AMI_CODESET;
+
 #endif /* __AMITYPE_H */
