@@ -22,7 +22,7 @@
 #include "ami-d.xpm"
 
 
-/*   */
+/* Common variables */
 static GtkWidget *d_area;
 static GdkPixbuf *pixbuf_h=NULL, *pixbuf_e=NULL, *pixbuf_d=NULL;
 static GdkFilterReturn e_event_translate(XEvent *xev, GdkEvent *ev, gpointer data);
@@ -38,7 +38,7 @@ static void wm_change_shape_mask(GtkWidget *win, GdkPixbuf *pixbuf);
 static void expose_cb(GtkWidget *w, GdkEventExpose *event, gpointer data);
 
 
-/*  Լ */
+/* Common functions */
 void 
 draw_applet_image(gint hangul_mode)
 {
@@ -47,11 +47,11 @@ draw_applet_image(gint hangul_mode)
       static int old_mode = -3;
       if ((!unique_han && support_status) || old_mode == hangul_mode) return;
       if (hangul_mode == AMI_PIXMAP_ENGLISH) {
-	 gdk_window_set_icon_name(cp_win->window, ": ƹ");
+	 gdk_window_set_icon_name(cp_win->window, _("ENG: Ami"));
       } else if (hangul_mode == AMI_PIXMAP_HANGUL) {
-	 gdk_window_set_icon_name(cp_win->window, "ѱ: ƹ");
+	 gdk_window_set_icon_name(cp_win->window, _("KOR: Ami"));
       } else {
-	 gdk_window_set_icon_name(cp_win->window, "ƹ ");
+	 gdk_window_set_icon_name(cp_win->window, _("Ami Preference"));
       }
       old_mode = hangul_mode;
       return;
@@ -112,8 +112,14 @@ get_pixbuf(int type)
 		break;
 	}
 	pixbuf = gdk_pixbuf_new_from_file(file);
+	if (pixbuf) {
+	    g_free(file);
+	    return pixbuf;
+	}
+
+	g_warning(_("Cannot create pixbuf using '%s'. Use internal pixmap"),
+		  file);
 	g_free(file);
-	return pixbuf;
     } 
 
     switch(type) {
@@ -134,7 +140,7 @@ get_pixbuf(int type)
 static GdkPixbuf *
 fit_size(GdkPixbuf *pixbuf, int size)
 {
-    if (size <= 0) return pixbuf;
+    if (size <= 0 || pixbuf == NULL) return pixbuf;
     if (size != gdk_pixbuf_get_width(pixbuf)
        || size != gdk_pixbuf_get_height(pixbuf)) {
     	GdkPixbuf *new_pixbuf = gdk_pixbuf_scale_simple(pixbuf, size, size, 
@@ -178,23 +184,24 @@ applet_change_pixmap_dir(char *dir_name)
 
 static void about (AppletWidget *applet, gpointer data)
 {
-  static const char *authors[] = { "Ȳ ġ", NULL};
-  GtkWidget *about_box;
+    static const char *authors[] = { N_("HWANG ChiDeok"), NULL};
+    GtkWidget *about_box;
 
-  about_box = gnome_about_new("ƹ ø", VERSION,
-	  "Copyright (C) Ȳ ġ 1999,2000,2001",
-	  authors,
-	  "ѱԷ± ƹ ׳ ø." ,NULL);
+    about_box = gnome_about_new(_("Ami Applet"), VERSION,
+			    _("Copyright (C) HWANG ChiDeok 1999,2000,2001"),
+			    authors,
+			    _("Gnome applet for Korean Input Server Ami"),
+			    NULL);
 
-  gtk_widget_show(about_box);
+    gtk_widget_show(about_box);
 }
 
 
 static gint 
 prop_destroy_cb( GtkWidget *widget, void *data )
 {
-  cp_win = NULL;
-  return FALSE;
+    cp_win = NULL;
+    return FALSE;
 }
 
 static void 
@@ -203,13 +210,13 @@ properties (AppletWidget *applet, gpointer data)
    GtkWidget *box, *vbox, *label, *frame;
   
    if( cp_win ){
-     gtk_widget_show(cp_win);
-     gdk_window_raise(cp_win->window);
-     return;
+       gtk_widget_show(cp_win);
+       gdk_window_raise(cp_win->window);
+       return;
    }
 
   cp_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title(GTK_WINDOW(cp_win), "ƹ ");
+  gtk_window_set_title(GTK_WINDOW(cp_win), _("Ami Preference"));
   gtk_container_add(GTK_CONTAINER(cp_win), cp);
 
   gtk_signal_connect( GTK_OBJECT(cp_win),
@@ -225,7 +232,7 @@ ami_gnome_main(int argc, char *argv[], char* rc_file)
   GdkBitmap *mask;
   gint i, j;
   
-  /* applet_widget_init  ϴ argv  */
+  /* get rid of arguments not recognized by applet_widget_init */
   for (i = 1; i < argc; i++) {
       if (!strcmp(argv[i], "-debug") || !strcmp(argv[i], "-gnome") || 
           !strcmp(argv[i], "-wm")) {
@@ -248,10 +255,10 @@ ami_gnome_main(int argc, char *argv[], char* rc_file)
 
   applet_widget_register_stock_callback (APPLET_WIDGET (applet),
 	  "about", GNOME_STOCK_MENU_ABOUT,
-	  " α׷...", about, NULL);
+	  _("About..."), about, NULL);
   applet_widget_register_stock_callback (APPLET_WIDGET (applet),
-	  "properties", GNOME_STOCK_MENU_ABOUT,
-	  "Ӽ...", properties, NULL);
+	  "properties", GNOME_STOCK_MENU_PROP,
+	  _("Properties..."), properties, NULL);
   d_area = gtk_drawing_area_new();
   gtk_widget_set_app_paintable(d_area,TRUE);
   gtk_widget_set_usize(d_area, 10, 10);
@@ -294,14 +301,14 @@ wm_destroy_cb( GtkWidget *widget, void *data )
 static void 
 wm_button_press_cb(GtkWidget * w, GdkEventButton * event, gpointer data)
 {
-    if(event->type != GDK_2BUTTON_PRESS) return; /*  Ŭ */
+    if(event->type != GDK_2BUTTON_PRESS) return; /* only process double-click */
     if(cp_win) {
 	gtk_widget_show(cp_win);
 	gdk_window_raise(cp_win->window);
 	return;
     }
     cp_win = gtk_window_new(GTK_WINDOW_DIALOG);
-    gtk_window_set_title(GTK_WINDOW(cp_win), "ƹ ");
+    gtk_window_set_title(GTK_WINDOW(cp_win), _("Ami Preference"));
     gtk_container_add(GTK_CONTAINER(cp_win), cp);
     gtk_signal_connect(GTK_OBJECT(cp_win), "delete_event", GTK_SIGNAL_FUNC(cp_delete_cb), NULL);
     gtk_widget_show_all(cp_win);
@@ -324,6 +331,8 @@ wm_change_shape_mask(GtkWidget *win, GdkPixbuf *pixbuf)
     GdkGC *gc;
     GdkColor color;
     int size = 64;
+
+    if (pixbuf == NULL) return;
     if (wm_type == WM_KDE1 || wm_type == WM_KDE2) {
 	if (icon_size <= 0) return;
 	size = icon_size;
@@ -427,8 +436,8 @@ ami_wm_main(int argc, char *argv[], char *rc_file)
     wm_change_shape_mask(icon_win, pixbuf_d);
 
     if (wm_type == WM_E) {
-	/* E  dock  */
-	/* E WindowMaker ſ ٸ  dock mechanism Ѵ.*/
+	/* setting up dock for E  which employs a different dock mechanism
+	   from that of Windowmaker */
 	gdk_window_add_filter(NULL, (GdkFilterFunc)e_event_translate, NULL); 
 	hints.initial_state = WithdrawnState;
 	hints.window_group = GDK_WINDOW_XWINDOW(icon_win->window);
@@ -479,19 +488,19 @@ e_event_translate(XEvent *xev, GdkEvent *ev, gpointer data)
 {
     if (xev->xany.send_event && xev->type == ButtonPress && 
     	gdk_window_lookup(xev->xbutton.window) == NULL) {
-	/* E icon window button pressȣ ؼ ڽ window .
-	   ڽſ ʿ  ̰ SendEvent  츮
-	   event  ̶ xbutton window  ״
-	    gtk+ ڱⰡ   event Ƿ
-	    ۵ ϰԵȴ.
-	    츮 ⿡ window ùٸ Ƽ ٽ
-	   X ۷ .
-	   GDK_FILTER_TRANSLATE window translation Ƿ
-	    
-	   gdk_event_put  double click ʱ⶧ Ұ.
-	   wmtimeͰ α׷  E ۵ϴ  
-	   windowü ˻ʰ ״ callback
-	   θǷ ̷  ߻ʾҴ.*/
+	/* To get a 'button press' signal, E creates a window on top of 
+	 * the icon window.  After extracting information they need, 
+	 * it informs us of the 'event' with |SendEvent|, but 
+	 * 'window info' of xbutton sent by E is not updated and  gtk+
+	 * doesn't work as intended because the event comes up 
+	 * to a window not created by gtk+. To work around this problem,
+	 * we manually reset |window| and send the event back to X buffer.
+	 * GDK_FILTER_TRANSLATE cannot be used as it doesn't support
+	 * window translation. Neither can gdk_event_put because double
+	 * click event is not supported. In case you wonder why wmtime
+	 * works fine under E, it does because it doesn't inspect
+	 * |window| but directly invokes callback.
+	 */
         xev->xbutton.window = GDK_WINDOW_XWINDOW(d_area->window);
         XPutBackEvent(gdk_display, xev);
 	return GDK_FILTER_REMOVE;
@@ -515,7 +524,7 @@ kde_is_running(void)
     if (XGetWindowProperty(gdk_display, gdk_root_window, en, 0, 256, False, en,
     	&type, &format, &nitems, &after, &root_data) != Success) return FALSE;
     if (!root_data) return FALSE;
-    if (*root_data) {
+    if (*(GdkAtom *) root_data) {
 	XFree(root_data);
 	return TRUE;
     } else {
@@ -540,7 +549,7 @@ kde2_is_running(void)
     if (XGetWindowProperty(gdk_display, gdk_root_window, en, 0, 256, False, en,
     	&type, &format, &nitems, &after, &root_data) != Success) return FALSE;
     if (!root_data) return FALSE;
-    if (*root_data) {
+    if (*(GdkAtom *) root_data) {
 	XFree(root_data);
 	return TRUE;
     } else {
