@@ -1,8 +1,10 @@
+/* This file is a part of Ami software, (C) Hwang chi-deok 1999, 2001 */
 
 #include "config.h"
 #include "edit.h"
 #include "ami.h"
 #include "util.h"
+#include "utf8.h"
 
 void
 cb_status_draw(IC *ic)
@@ -28,14 +30,24 @@ cb_status_draw(IC *ic)
     if(debug) g_print("%s: %s\n", __FUNCTION__, txt);
 }
 
+int pop_mbc(unsigned char *d, char *stack)
+{
+    int len=MBC_LEN_B(stack-1); 
+    strncpy(d,stack-len,len);
+    d[len]='\0';
+    return len;
+}
+
+
 void
 cb_edit_clear_temp(IC *ic)
 {
     IMPreeditCBStruct data;
     XIMText text;
     XIMFeedback feedback[2];
-    char tmpbuf[3];
+    char tmpbuf[4];
     char *cts;
+
     data.major_code = XIM_PREEDIT_DRAW;
     data.connect_id = ic->connect_id;
     data.icid = ic->id;
@@ -47,9 +59,9 @@ cb_edit_clear_temp(IC *ic)
     feedback[0] = XIMUnderline;
     feedback[1] = 0;
     text.feedback = feedback;
-    tmpbuf[0] = ic->han->buf[ic->han->pos-2];
-    tmpbuf[1] = ic->han->buf[ic->han->pos-1];
-    tmpbuf[2] = '\0';
+
+    pop_mbc((unsigned char*) tmpbuf, ic->han->buf+ic->han->pos); 
+
     cts = editing_mbstocts(tmpbuf);
     text.string.multi_byte = cts;
     text.length = strlen(cts);
@@ -66,6 +78,7 @@ cb_edit_correct_insert(IC *ic, char *buf)
     XIMFeedback feedback[3];
     char *cts;
     int pos;
+
     if (debug) g_print("%s\n",__FUNCTION__);
 
     data.major_code = XIM_PREEDIT_DRAW;
@@ -123,7 +136,10 @@ cb_edit_insert(IC *ic, char *buf)
     char *cts;
     XIMFeedback feedback[3];
     int pos;
-    g_assert (buf[2] == '\0');
+    int len = MBC_LEN(buf[0]);
+
+    g_assert (buf[len] == '\0');
+
     if (debug) g_print("%s\n",__FUNCTION__);
 
     data.major_code = XIM_PREEDIT_DRAW;
@@ -135,15 +151,16 @@ cb_edit_insert(IC *ic, char *buf)
     text.feedback = feedback;
 
     if (ic->han->pos >= 2) {
-	char tmpbuf[5];
+	unsigned char tmpbuf[9]; /* A char. in UTF-8 can take up up to 4 octets*/
+	int l,l1;
+
 	feedback[0] = XIMUnderline;
 	feedback[1] = XIMReverse;
 	feedback[2] = 0;
-	tmpbuf[0] = ic->han->buf[ic->han->pos-2];
-	tmpbuf[1] = ic->han->buf[ic->han->pos-1];
-	tmpbuf[2] = buf[0];
-	tmpbuf[3] = buf[1];
-	tmpbuf[4] = '\0';
+	l=pop_mbc(tmpbuf, ic->han->buf + ic->han->pos);
+	l1=MBC_LEN(buf[0]);
+	strncpy(tmpbuf+l,buf,l1);
+	tmpbuf[l+l1]='\0';
 	data.todo.draw.chg_length = 1;
 	data.todo.draw.chg_first = pos - 1;
 	cts = editing_mbstocts(tmpbuf);
