@@ -41,21 +41,30 @@ cli_win_monitor(XEvent *xev, GdkEvent *ev, gpointer data)
 void
 cli_win_register(GdkWindow *win)
 {
+    GdkWindowPrivate *private;
+
     g_return_if_fail(win != NULL);
+
     gdk_window_set_events(win, 0);
     gdk_window_add_filter(win, (GdkFilterFunc)cli_win_monitor, NULL); 
+
+    private = (GdkWindowPrivate *) win;
+    if (private->parent) {
+	GdkWindowPrivate *parent = (GdkWindowPrivate *) private->parent;
+	parent->children = g_list_remove(parent->children, win);
+	private->parent = NULL;
+    }
 }
 
 void
 clean_han_buf(IC *ic)
 {
     if (ic && ic_is_valid(ic) && ic->han) {
-	g_free(ic->han->buf);
 	if (ic->han->aux_input) {
 	    gtk_object_set_data((GtkObject *)ic->han->aux_input, "ic", NULL);
 	    gtk_widget_destroy(ic->han->aux_input);
 	}
-	g_free(ic->han);
+	ic_han_free(ic->han);
 	ic->han = NULL;
     }
 }
@@ -82,8 +91,8 @@ cli_get_edit_ic(IC *ic)
 {
     IC *edit_ic;
     g_return_val_if_fail (ic != NULL, NULL);
-    /* netscape  client_win ϱ⵵  ic
-       ֹ 찡 ִ */
+    /* Netscape (4.x?) sometimes loses ic even before setting up
+     * client_win. */ 
     if (ic->client_win == NULL) return NULL;
     g_return_val_if_fail (ic->top_win != NULL, NULL);
     edit_ic = g_dataset_get_data(ic->top_win, "edit_ic");
@@ -96,8 +105,8 @@ cli_get_status_ic(IC *ic)
 {
     IC *status_ic;
     g_return_val_if_fail (ic != NULL, NULL);
-    /* netscape  client_win ϱ⵵  ic
-       ֹ 찡 ִ */
+    /* Netscape (4.x?) sometimes loses ic even before setting up
+     * client_win. */ 
     if (ic->client_win == NULL) return NULL;
     g_return_val_if_fail (ic->top_win != NULL, NULL);
     status_ic = g_dataset_get_data(ic->top_win, "status_ic");
