@@ -1,4 +1,9 @@
 /* this file is a part of Ami software, (C) Hwang chi-deok 1999,2000,2001 */
+/* This file includes a couple of arrays initialized with 
+ * character strings in EUC-KR. If you don't have an editor that can
+ * deal with EUC-KR, you have to make sure that your editor does not
+ * haphazardly trashes the content.
+ */
 
 #include "config.h"
 
@@ -10,11 +15,8 @@
 #include "conf.h"
 #include "keyparse.h"
 
-#ifndef ENGLISH_KEYBOARD
-#define ENGLISH_KEYBOARD "1"
-#endif
 #ifndef HANGUL_KEYBOARD
-#define HANGUL_KEYBOARD "1"
+#define HANGUL_KEYBOARD "2bul"
 #endif
 
 static char * config_readline(FILE *fp);
@@ -23,10 +25,9 @@ static int save_default_config_file(void);
 static ConfigItem *ami_config = NULL;
 
 
-static ConfigDesc ami_config_type[] = 
+static ConfigDesc ami_config_type[] =
 {
-    {"ѱ ", INT, &ami_keyboard},
-    {" ", INT, &ami_ekeyboard},
+    {"", STRING, &keyboard_name},
     {"Է ", INT, &ami_editing_mode},
     {"⺻ ۲", STRING, &default_fontname},
     {"ѿ ȯŰ", KEYSYM, &trigger_keys},
@@ -46,6 +47,7 @@ static ConfigDesc ami_config_type[] =
     {"Ҹ ũ", INT, &bell_intensity},
     {"׸  ġ", STRING, &pix_dir_name},
     {"/߼ Է  ڹٲ ", BOOLEAN, &allow_shuffle},
+    {"۲ (UTF-8 locale ۲ )",INT,&num_fonts_in_utf8_fs},
     {NULL, 0, NULL}
 };
 
@@ -57,25 +59,27 @@ ami_config_init(void)
     char *conf_file = g_strconcat(g_get_home_dir(), "/.ami/ami.conf", NULL);
     if (access(conf_file, R_OK) != 0) {
 	if (!save_default_config_file())
-	    g_error("%s  ϴ", conf_file);
-    } 
+	    g_error(_("Cannot write to %s\n"), conf_file);
+    }
     ret = ami_config_load_file(conf_file, ami_config_type);
     on_keys = g_new(XIMTriggerKeys, 1);
     if (trigger_keys == NULL) {
-	g_error("ѿ ȯŰ Ǿ ʽϴ.");
+	g_error(_("No key is specified for Korean-English KBD toggle."));
     }
     for(i=0;trigger_keys[i].keysym;i++);
     on_keys->count_keys = i;
     on_keys->keylist = trigger_keys;
     g_free(conf_file);
     ami_hanja_subst_mode = ami_hanja_def_subst_mode;
-    if (!hangul_mode_label) 
-       hangul_mode_label = g_strdup(" [ѱ] ");
-    if (!english_mode_label) 
-       english_mode_label = g_strdup(" [] ");
-    if (!ami_hanja_subst_format) 
-       ami_hanja_subst_format = g_strdup(":():()");
+    if (!hangul_mode_label)
+	hangul_mode_label = g_strdup(_(" [KO] "));
+    if (!english_mode_label)
+	english_mode_label = g_strdup(_(" [EN] "));
+    if (!ami_hanja_subst_format)
+	ami_hanja_subst_format = g_strdup(":(ѱ):ѱ()");
     if (!hanja_dic_filename) hanja_dic_filename = g_strdup(HANJA_DIC);
+    if (!keyboard_name) keyboard_name = g_strdup(HANGUL_KEYBOARD);
+    composer_set_shuffle_active((int) allow_shuffle);
     return ret;
 }
 
@@ -108,7 +112,7 @@ save_default_config_file(void)
 {
     char *conf_dir = g_strdup_printf("%s/%s", g_get_home_dir(), ".ami");
     char *conf;
-    extern char *default_config;
+    extern const char *default_config;
     struct stat statbuf;
     FILE *fp;
     if (stat(conf_dir, &statbuf) != 0) {
@@ -118,7 +122,7 @@ save_default_config_file(void)
     g_free(conf_dir);
     fp = fopen(conf, "w");
     if (fp == NULL) {
-	g_error("ƹ  %s Ҽ ϴ.", conf);
+	g_error(_("Cannot save Ami configuration options to %s."), conf);
     }
     g_free(conf);
     fputs(default_config, fp);
@@ -159,8 +163,8 @@ config_add_item(guchar *desc, guchar *key, guchar *val_string)
 	    *config_item->val.integer = atoi(val_string);
 	    break;
 	case BOOLEAN:
-	    if (*val_string == 't' 
-	    	|| strcmp(val_string, "on") == 0 
+	    if (*val_string == 't'
+	    	|| strcmp(val_string, "on") == 0
 		|| *val_string == '0') *config_item->val.boolean = TRUE;
 	    else if(*val_string == 'f'
 	    	|| strcmp(val_string, "off") == 0
@@ -174,7 +178,7 @@ config_add_item(guchar *desc, guchar *key, guchar *val_string)
 	    break;
 	case STRING:
 	    if (*config_item->val.string) g_free(*config_item->val.string);
-	    if (val_string[0] == '"' && 
+	    if (val_string[0] == '"' &&
 	        val_string[strlen(val_string)-1] == '"') {
 		val_string[strlen(val_string)-1] = 0;
 		val_string++;
@@ -201,17 +205,17 @@ ami_config_load_file(char *file, ConfigDesc *ami_config_type)
 	ami_config[i].key = ami_config_type[i].key;
 	ami_config[i].type = ami_config_type[i].type;
 	switch(ami_config[i].type) {
-	    case STRING: 
-		ami_config[i].val.string = ami_config_type[i].val_pointer; 
+	    case STRING:
+		ami_config[i].val.string = ami_config_type[i].val_pointer;
 		break;
-	    case INT: 
-		ami_config[i].val.integer = ami_config_type[i].val_pointer; 
+	    case INT:
+		ami_config[i].val.integer = ami_config_type[i].val_pointer;
 		break;
-	    case KEYSYM: 
-		ami_config[i].val.keys = ami_config_type[i].val_pointer; 
+	    case KEYSYM:
+		ami_config[i].val.keys = ami_config_type[i].val_pointer;
 		break;
-	    case BOOLEAN: 
-		ami_config[i].val.boolean = ami_config_type[i].val_pointer; 
+	    case BOOLEAN:
+		ami_config[i].val.boolean = ami_config_type[i].val_pointer;
 		break;
 	}
     }
@@ -225,12 +229,12 @@ ami_config_load_file(char *file, ConfigDesc *ami_config_type)
 	    guchar * start, *end, *key, *val;
 	    GString *desc_string;
 	    if (sep == NULL) {
-		g_warning("invalid config line %s", line);
+		g_warning(_("invalid config line: %s"), line);
 		continue;
 	    }
 	    *sep = 0;
 	    start = line_skip_space(line);
-	    end = line_rskip_space(sep - 1); 
+	    end = line_rskip_space(sep - 1);
 	    if (isspace(*end)) *end = 0;
 	    else *(end+1) = 0;
 	    key = start;
@@ -250,7 +254,7 @@ ami_config_load_file(char *file, ConfigDesc *ami_config_type)
 	    g_slist_free(comment);
 	    comment = NULL;
 	    if (!config_add_item(desc_string->str, key, val)) {
-		g_warning("%s is not valid key", key);
+		g_warning(_("%s is not a valid key"), key);
 	    };
 	    g_string_free(desc_string, TRUE);
 	    g_free(line);
@@ -283,7 +287,7 @@ ami_config_save_file(char *file)
 		fprintf (fp, "%s=%d\n", config->key, *config->val.integer);
 		break;
 	    case BOOLEAN:
-		if(*config->val.boolean) 
+		if(*config->val.boolean)
 		    fprintf (fp, "%s=true\n", config->key);
 		else
 		    fprintf (fp, "%s=false\n", config->key);
@@ -303,130 +307,146 @@ ami_config_save_file(char *file)
 	}
 	config++;
     }
+    fprintf(fp, "\n");
     fclose(fp);
     return TRUE;
 }
 
-
-char *default_config = 
-"
-# ƹ 
-# :  Ű  ٲʽÿ.  \"ѱ \"̶ Ǿ
-#   ؼ Ȯ ̸̾մϴ.
-
-# # ѱ Ű 
-# 1 = ȯ溯(AMI_KEYBOARD, HANGUL_KEYBOARD_TYPE)κ 
-# 2 = ι, 3 =  390, 4 =  
-
-ѱ  = " HANGUL_KEYBOARD "
-
-# #  Ű 
-# 1 = qwerty, 2 = Dvorak 
-
-  = " ENGLISH_KEYBOARD "
-
-# Է½  Ƽ   ݴϴ.
-# 1=ܾ ϸ Է  ȭǥŰ ̿Ͽ ̵ մϴ.
-# 2=ܾ ϸ ȭǥŰ ̿ ̵ α׷  
-#   ʿ ó ϵ մϴ. ϴ Դϴ.
-# 3=ڴ  մϴ.
-
-Է  = 1
-
-#  α׷  ѿ¸  Ұ 
-# false: ѿ° â ˴ϴ. XIM ǥؿ  յǴ Դϴ.
-#  httͰ α׷ ޸ program ִ  ʰ
-#   â ؼ gtk+Ͱ α׷   ϵ
-#  մϴ.
-# true:  α׷  ѿ¸ Ե˴ϴ. â ʽϴ.
-#   ٸ     մϴ.
-
- ѿ = false
-
-
-# ѿȯ  Ǵ Ű ݴϴ
-
-ѿ ȯŰ = Shift<Key>space, <Key>Hangul
-
-#  Է Ű
-
- ȯŰ = <Key>F9, <Key>Hangul_Hanja
-
-# Ư  ԷŰ 
-
-Ư ԷŰ = <Key>F3
-
-#  Է Էâ  α׷   Ű
-
-flush key = Ctrl<Key>Return
-
-# â ȭ  Ʒκп Ÿ (Ǵ gtk entryͰ ¥ 
-# Էâ) Էâ  ʱ ѱԷ  Ǿ   Է
-# ȭ  κп ̷ ƴϸ  Էµǰִ  
-# ũ ų Ÿ մϴ.
-# true̸ ׳ ũ ŵϴ.
-
-last line scroll = true
-
-# ⺻ ۲ 
-# ƹ̿ ϴ α׷ ۲ ʰų ߸ ̸ 쿡
-#  ۲ ֽø ˴ϴ.  ۲,ѱ۱۲
-
-⺻ ۲ = *,*
-
-# ѱ  ϶ â ǥõǴ ڿ ݴϴ.
-
-ѱۻ  = \" [ѱ] \"
-
-#  Է ϶ â ǥõǴ ڿ ݴϴ.
-
-  = \" [] \"
-
-# ѱԷ ¿ Escape  · ڵ 
-# ݴϴ. true/false
-
-escape hangul toggle = true
-
-# â  ݴϴ.
-# â Ʒʿ   â 彺 е  ɼ 
-# false Ͻʽÿ. 
-#  ѱۻ¸ Ͻø  ɼǰ  â ʽϴ.
-
-â  = true
-
-# ڸ ġȯϴ ¸ ݴϴ.
-#  Ǿִ κ   ڷ ġȯǰ \"\"
-#  κ  ڿ شϴ ѱ ־Ե˴ϴ.
-# µ  \":\" մϴ.
-
- ġȯ  = :():()
-
-# ڸ ġȯ¿  °  ⺻ Ұ
-# մϴ.
-# 1 
-
- ġȯ  = 1
-
-#  Էâ  ִ ѱ ǥ  Ʒʿ   
-# ƴϸ   â  ٸ  
-
-  = true
-
-#   ġ 
-
-ڻ = " HANJA_DIC "
-
-#  Ҹ ũ⸦ ݴϴ.
-# ȿ  -100 100̰  Ŭ Ҹ Ŀϴ.
-Ҹ ũ = 0
-
-# ѿ ǥø  ׸ ġϴ 丮 մϴ.
-# "" Ǿ  ƹ̿  ׸ ϰԵ˴ϴ.
-# 丮̸ '/' ϴ 丮 ̸   ּžմϴ.
-׸  ġ = ""
-
-# ʼ ߼ Է ʼ   Ÿ  ٲ 
-# ϰ ʼ+߼  Է  
-/߼ Է  ڹٲ  = false
-
+/*
+   '\n\' has to be used to make xgettext not complain about unterminated
+  string literals. J. Shin
+*/
+const char *default_config = 
+"\n\
+# Caution: This file MUST be in EUC-KR encoding. If you use UTF-8 locale,\n\
+# convert this file to UTF-8 before editing and convert back to EUC-KR\n\
+# after you're done with editing. Otherwise, Ami wouldn't be able to\n\
+# parse the file to get the values of configuration options.\n\
+# ƹ  \n\
+# :  Ű  ٲ ʽÿ.  \"Է \"̶ Ǿ\n\
+#   ؼ Ȯ ̸̾ մϴ.\n\
+\n\
+# \n\
+# 2bul, 3bul_390, 3bul_final, dvorak_2bul, dvorak_3bul_390,\n\
+# dvorak_3bul_final Ami ǿ  ִ  Դϴ.\n\
+# ߰ ٸ  ؼ   ֽϴ.\n\
+=" HANGUL_KEYBOARD "\n\
+\n\
+# Է½  Ƽ   ݴϴ.\n\
+# 1=ܾ  ϸ Է  ȭǥ Ű ̿ ̵ Ami \n\
+#   óմϴ.\n\
+# 2=ܾ  ϸ ȭǥ Ű ̿ ̵ Ŭ̾Ʈ α׷ \n\
+#   Ͽ  ó ϵ մϴ. ϴ Դϴ.\n\
+# 3=  մϴ.\n\
+\n\
+Է  = 2\n\
+\n\
+#  Ŭ̾Ʈ α׷  ѿ ¸   \n\
+# false: ѿ ¸ â  մϴ. XIM ǥؿ  ϴ Դϴ.\n\
+#  htt  Է±ʹ ޸ Ŭ̾Ʈ ִ   ʰ\n\
+#   â ؼ gtk+  α׷   ϵ\n\
+#  մϴ.\n\
+# true:  Ŭ̾Ʈ α׷  ѿ ¸ ϴ. â \n\
+#  ʽϴ.    ٸ     մϴ.\n\
+\n\
+ ѿ = false\n\
+\n\
+\n\
+# ѿ ȯ  Ǵ Ű ݴϴ\n\
+\n\
+ѿ ȯŰ = Shift<Key>space, <Key>Hangul\n\
+\n\
+#  Է Ű\n\
+\n\
+ ȯŰ = <Key>F9, <Key>Hangul_Hanja\n\
+\n\
+# Ư  ԷŰ \n\
+\n\
+Ư ԷŰ = <Key>F3\n\
+\n\
+#  Է  Էâ  α׷   Ű\n\
+\n\
+flush key = Ctrl<Key>Return\n\
+\n\
+# â ȭ  Ʒ κп Ÿ (Ǵ gtk entry   ¥ \n\
+# Էâ) Էâ  ʱ ѱ Է  Ǿ   Է\n\
+# ȭ   κп ̷  ƴϸ  Էµǰ ִ  \n\
+#  ũ ų   մϴ.\n\
+# true̸ ׳ ũ ŵϴ.\n\
+\n\
+last line scroll = true\n\
+\n\
+# ⺻ ۲ \n\
+# ƹ̿ ϴ α׷ ۲   ʰų ߸ ̸ 쿡\n\
+#  ۲ ֽø ˴ϴ.   ۲,ѱ ۲\n\
+# Ami UTF-8 locale ٸ Ʒ     Դϴ.\n\
+# ׷ ʰ EUC-KR locale ٸ  Ʒ  ᵵ  , \n\
+# \"*,*\"  ־ ϴ.\n\
+\n\
+⺻ ۲ = -*-*-medium-r-normal-ko-16-*-*-*-*-*-iso10646-1,-*-*-medium-r-normal--16-*-*-*-*-*-*-*\n\
+\n\
+# ѱ    â ǥõǴ ڿ ݴϴ.\n\
+\n\
+ѱۻ  = \" [KO] \"\n\
+\n\
+#  Է   â ǥõǴ ڿ  ݴϴ.\n\
+\n\
+  = \" [EN] \"\n\
+\n\
+# ѱ Է ¿ Escape    · ڵ  \n\
+# ݴϴ. true/false\n\
+\n\
+escape hangul toggle = true\n\
+\n\
+# â    ݴϴ.\n\
+# â Ʒʿ   â 彺 е  ɼ \n\
+# false Ͻʽÿ. \n\
+#  ѱ ¸ Ͻø  ɼǰ   â  ʽϴ.\n\
+\n\
+â  = true\n\
+\n\
+# ڸ ġȯϴ ¸ ݴϴ.\n\
+# \"\" Ǿִ κ   ڷ ġȯǰ \"ѱ\"̶\n\
+#  κ  ڿ شϴ ѱ ־ ˴ϴ.\n\
+# µ   \":\" մϴ.\n\
+\n\
+ ġȯ  = :(ѱ):ѱ()\n\
+\n\
+#  ġȯ ¿  °  ⺻  \n\
+# մϴ.\n\
+# 1 \n\
+\n\
+ ġȯ  = 1\n\
+\n\
+#  Էâ  ִ ѱ ǥ  Ʒʿ   \n\
+# ƴϸ   â  ٸ   \n\
+\n\
+  = true\n\
+\n\
+#   ġ \n\
+\n\
+ڻ = " HANJA_DIC "\n\
+\n\
+#  Ҹ ũ⸦ ݴϴ.\n\
+# ȿ  -100 100̰  Ŭ Ҹ Ŀϴ.\n\
+Ҹ ũ = 0\n\
+\n\
+# ѿ ǥø  ׸  ġϴ 丮 մϴ.\n\
+# \"\" Ǿ  ƹ̿  ׸ մϴ.\n\
+# 丮̸ '/' ϴ 丮 ̸   ּž մϴ.\n\
+׸  ġ = \"\"\n\
+\n\
+# ʼ  ߼ Է  ʼ   Ÿ  ٲ  \n\
+# ϰ ʼ+߼  Է  \n\
+/߼ Է  ڹٲ  = false\n\
+\n\
+# UTF-8 locale   ۲ (fontset) ʿ ϴ\n\
+# ۲  մϴ. $XROOT/locale/ko_KR.UTF-8/XLC_LOCALE\n\
+#  ۲  ⿡  ˴ϴ. XFree86 4.x ִ\n\
+# en_US.UTF-8/XLC_LOCALE 7 ۲ Ǿ ֽϴ.\n\
+# ݸ鿡 X11R6.6 Solaris 8 Ȥ 9 ko_KR.UTF-8/XLC_LOCALE \n\
+#  18 ۲ Ǿ ֽϴ. \n\
+\n\
+۲ (UTF-8 locale ۲ )= 7\n\
+\n\
 ";
