@@ -1,12 +1,14 @@
-/* this file is a part of Ami software, (C) Hwang chi-deok 1999, 2000, 2001 */
+/* this file is a part of Ami software, (C) Hwang chi-deok 1999, 2000, 2001, 2002 */
 
 #include "config.h"
 #include "cp.h"
 #include "ami.h"
 #include "hinput.h"
 #include "conf.h"
+#include "util.h"
 #include "keyparse.h"
 #include "applet.h"
+#include "comp.h"
 #include <dirent.h>
 
 static GList *activate_list = NULL;
@@ -17,22 +19,14 @@ static gint cp_delete( GtkWidget *widget, void *data );
 static GtkTooltips *tooltip = NULL;
 static guint update_id = 0;
 
+
+
 static void
 keyboard_toggle_cb(GtkWidget *w, gpointer data)
 {
     if (GTK_TOGGLE_BUTTON(w)->active) {
-	ami_keyboard = GPOINTER_TO_INT(data);
-	ami_hangul_keyboard_set(ami_keyboard);
-    }
-}
-
-static void
-ekeyboard_toggle_cb(GtkWidget *w)
-{
-    if (GTK_TOGGLE_BUTTON(w)->active) {
-	ami_ekeyboard = 1;
-    } else {
-	ami_ekeyboard = 2;
+	keyboard_name = data;
+	composer_set_keymap(keyboard_name);
     }
 }
 
@@ -62,12 +56,13 @@ static void
 shuffle_mode_toggle_cb(GtkWidget *w, gpointer data)
 {
     allow_shuffle = GTK_TOGGLE_BUTTON(w)->active;
+    composer_set_shuffle_active(allow_shuffle);
 }
 
 static void
 unique_han_toggle_cb(GtkWidget *w, gpointer data)
 {
-    warn_no_effect("ѱ   ƹ̸ ٽ ž߸ ȿ ֽϴ.");
+    warn_no_effect(_("This change will get effective only after you relaunch Ami."));
     unique_han = GTK_TOGGLE_BUTTON(w)->active;
 }
 
@@ -90,14 +85,14 @@ key_setting_activate(GtkWidget *w, char *kind)
     if (!gtk_object_get_data(GTK_OBJECT(w), "changed")) return;
     gtk_object_set_data(GTK_OBJECT(w), "changed", NULL);
     key = gtk_entry_get_text(GTK_ENTRY(w));
-    if (!strcmp(kind, "ѿ")) {
+    if (!strcmp(kind, "KorEng")) {
 	make_trigger_keys(&trigger_keys, key);
-	warn_no_effect("ѿȯŰ  ٽ ƹ̸ ϼž߸ ȿ ֽϴ.");
-    } else if (!strcmp(kind, "")) {
+	warn_no_effect(_("The toggle key for Korean and English kbd. will get effective only after relaunching Ami."));
+    } else if (!strcmp(kind, "Hanja")) {
 	make_trigger_keys(&hanja_keys, key);
-    } else if (!strcmp(kind, "Ư")) {
+    } else if (!strcmp(kind, "SpecialChar")) {
 	make_trigger_keys(&special_char_keys, key);
-    } else if (!strcmp(kind, " ")) {
+    } else if (!strcmp(kind, "BufferClean")) {
 	make_trigger_keys(&flush_keys, key);
     }
 }
@@ -114,7 +109,7 @@ label_setting_activate(GtkEntry *w, gchar **label)
 static void
 support_status_toggled(GtkWidget *w)
 {
-    warn_no_effect("â  ƹ̸ ٽ ž߸ ȿ ֽϴ.");
+    warn_no_effect(_("En/Disabling the status window gets effective only after relaunching Ami."));
     support_status = GTK_TOGGLE_BUTTON(w)->active;
 }
 
@@ -141,82 +136,33 @@ save_conf(GtkWidget *w, gpointer data)
     ami_config_save_file(NULL);
 }
 
-void
-set_keyboard(void)
-{
-    ami_hangul_keyboard_set(ami_keyboard);
-}
-
 static GtkWidget *
 create_keyboard_setting(void)
 {
-    /* Ű  */
-    GtkWidget *vbox0, *vbox1, *frame, *hbox;
-    GtkWidget *qwerty, *dvorak;
-    GtkWidget *button0, *button1, *button2, *button3;
+    /* set up a keyboard */
+    GtkWidget *vbox;
+    GtkWidget *button;
     GSList *group;
     int keyboard;
+    char **kbd_names;
+    int num_kbd;
+    int i;
 
-    vbox0 = gtk_vbox_new(FALSE, 2);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox0), 5);
-
-    frame = gtk_frame_new("ѱ ");
-    gtk_box_pack_start(GTK_BOX(vbox0), frame, FALSE, FALSE, 5);
-
-    vbox1 = gtk_vbox_new(TRUE, 2);
-    gtk_container_add(GTK_CONTAINER(frame), vbox1);
-    button0 = gtk_radio_button_new_with_label(NULL, "ȯ溯");
-    group = gtk_radio_button_group(GTK_RADIO_BUTTON(button0));
-    gtk_container_add(GTK_CONTAINER(vbox1), button0);
-
-    button1 = gtk_radio_button_new_with_label(group, "ι");
-    group = gtk_radio_button_group(GTK_RADIO_BUTTON(button1));
-    gtk_container_add(GTK_CONTAINER(vbox1), button1);
-
-    button2 = gtk_radio_button_new_with_label(group, " 390");
-    group = gtk_radio_button_group(GTK_RADIO_BUTTON(button2));
-    gtk_container_add(GTK_CONTAINER(vbox1), button2);
-
-    button3 = gtk_radio_button_new_with_label(group, " ");
-    gtk_container_add(GTK_CONTAINER(vbox1), button3);
-
-/*    keyboard = ami_hangul_keyboard_get(); */
-    switch(ami_keyboard) {
-	case 2:
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button1), TRUE);
-	    break;
-	case 3:
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button2), TRUE);
-	    break;
-	case 4:
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button3), TRUE);
-	    break;
-	default:
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button0), TRUE);
-	    break;
+    vbox = gtk_vbox_new(FALSE, 2);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
+    kbd_names = composer_get_keyboard_list(KEYBOARD_DIR, &num_kbd);
+    composer_set_keymap(keyboard_name);
+    group = NULL;
+    for(i=0;i<num_kbd;i++) {
+	button = gtk_radio_button_new_with_label(group, kbd_names[i]);
+	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 4);
+	group = gtk_radio_button_group(GTK_RADIO_BUTTON(button));
+	if (strcmp(keyboard_name, kbd_names[i]) == 0)
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
+	gtk_signal_connect(GTK_OBJECT(button), "toggled", 
+	    (GtkSignalFunc)keyboard_toggle_cb, kbd_names[i]);
     }
-    gtk_signal_connect(GTK_OBJECT(button0), "toggled", (GtkSignalFunc)keyboard_toggle_cb, GINT_TO_POINTER(1));
-    gtk_signal_connect(GTK_OBJECT(button1), "toggled", (GtkSignalFunc)keyboard_toggle_cb, GINT_TO_POINTER(2));
-    gtk_signal_connect(GTK_OBJECT(button2), "toggled", (GtkSignalFunc)keyboard_toggle_cb, GINT_TO_POINTER(3));
-    gtk_signal_connect(GTK_OBJECT(button3), "toggled", (GtkSignalFunc)keyboard_toggle_cb, GINT_TO_POINTER(4));
-
-    frame = gtk_frame_new(" ");
-    gtk_box_pack_start(GTK_BOX(vbox0), frame, FALSE, FALSE, 5);
-
-    hbox = gtk_hbox_new(TRUE, 2);
-    gtk_container_add(GTK_CONTAINER(frame), hbox);
-    qwerty = gtk_radio_button_new_with_label(NULL, "Qwerty");
-    gtk_container_add(GTK_CONTAINER(hbox), qwerty);
-    dvorak = gtk_radio_button_new_with_label(
-	gtk_radio_button_group(GTK_RADIO_BUTTON(qwerty)), "Dvorak");
-    gtk_container_add(GTK_CONTAINER(hbox), dvorak);
-    if (ami_ekeyboard == 1) {
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(qwerty), TRUE);
-    } else {
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dvorak), TRUE);
-    }
-    gtk_signal_connect(GTK_OBJECT(qwerty), "toggled", (GtkSignalFunc)ekeyboard_toggle_cb, NULL);
-    return vbox0;
+    return vbox;
 }
 
 static GtkWidget *
@@ -228,24 +174,25 @@ create_input_setting(void)
     GtkWidget *b;
     GSList *group;
     gchar **hanja_formats;
+    GString *fmts_xlat;
     int i;
 
     vbox0 = gtk_vbox_new(FALSE, 2);
     gtk_container_set_border_width(GTK_CONTAINER(vbox0), 5);
 
-    frame = gtk_frame_new("Է ");
+    frame = gtk_frame_new(_("Input Method"));
     gtk_box_pack_start(GTK_BOX(vbox0), frame, FALSE, FALSE, 0);
 
     vbox1 = gtk_vbox_new(TRUE, 2);
     gtk_container_add(GTK_CONTAINER(frame), vbox1);
 
-    wbutton = gtk_radio_button_new_with_label(NULL, "ܾ: ̵");
+    wbutton = gtk_radio_button_new_with_label(NULL, _("Commit by words: Cursor movement processed by Ami"));
     gtk_container_add(GTK_CONTAINER(vbox1), wbutton);
     swbutton = gtk_radio_button_new_with_label(
-	gtk_radio_button_group(GTK_RADIO_BUTTON(wbutton)), "ܾ: ̵Ұ");
+	gtk_radio_button_group(GTK_RADIO_BUTTON(wbutton)), _("Commit by words: Cursor movement delegated to clients"));
     gtk_container_add(GTK_CONTAINER(vbox1), swbutton);
     cbutton = gtk_radio_button_new_with_label(
-	gtk_radio_button_group(GTK_RADIO_BUTTON(swbutton)), "ڴ");
+	gtk_radio_button_group(GTK_RADIO_BUTTON(swbutton)), _("Commit by characters"));
     gtk_container_add(GTK_CONTAINER(vbox1), cbutton);
     if (ami_editing_mode == AMI_EDITING_WORD_MODE) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wbutton), TRUE);
@@ -259,12 +206,13 @@ create_input_setting(void)
     gtk_signal_connect(GTK_OBJECT(swbutton), "toggled", (GtkSignalFunc)editing_mode_toggle_cb, GINT_TO_POINTER(AMI_EDITING_SIMPLE_WORD_MODE));
     gtk_signal_connect(GTK_OBJECT(cbutton), "toggled", (GtkSignalFunc)editing_mode_toggle_cb, GINT_TO_POINTER(AMI_EDITING_CHAR_MODE));
 
-    frame = gtk_frame_new(" ġȯ ");
+    frame = gtk_frame_new(_("Hanja Conversion Mode"));
     gtk_box_pack_start(GTK_BOX(vbox0), frame, FALSE, FALSE, 0);
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(frame), hbox);
 
-    hanja_formats = g_strsplit(ami_hanja_subst_format, ":", -1);
+    fmts_xlat = util_xlate_hanja_fmts();
+    hanja_formats = g_strsplit(fmts_xlat->str, ":", -1);
     group = NULL;
     for(i=0;hanja_formats[i] != NULL; i++) {
 	b = gtk_radio_button_new_with_label(group, hanja_formats[i]);
@@ -277,28 +225,30 @@ create_input_setting(void)
 		GINT_TO_POINTER(i+1));
     }
     g_strfreev(hanja_formats);
+    g_string_free(fmts_xlat, TRUE);
 
-    esc = gtk_check_button_new_with_label("Escape ѱ ȯ");
+    esc = gtk_check_button_new_with_label(_("On <ESC>, switch to English mode"));
     gtk_box_pack_start(GTK_BOX(vbox0), esc, FALSE, FALSE, 5);
     if (esc_han_toggle) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(esc), TRUE);
     }
     gtk_signal_connect(GTK_OBJECT(esc), "toggled", (GtkSignalFunc)esc_mode_toggle_cb, NULL);
 
-    esc = gtk_check_button_new_with_label(" ѿ ");
+    esc = gtk_check_button_new_with_label(_("Keep KOR/ENG mode across clients"));
     gtk_box_pack_start(GTK_BOX(vbox0), esc, FALSE, FALSE, 5);
     if (unique_han) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(esc), TRUE);
     }
     gtk_signal_connect(GTK_OBJECT(esc), "toggled", (GtkSignalFunc)unique_han_toggle_cb, NULL);
-    gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), esc, "ѱ   ƹ̸ ٽ ž߸ ȿ ֽϴ.", NULL);
+    gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), esc, _("To get this change effective, you have to relaunch Ami."), NULL);
 
-    esc = gtk_check_button_new_with_label("/߼ ڹٲ ");
+    esc = gtk_check_button_new_with_label(_("Intelligent handling of misplaced Jamos"));
     gtk_box_pack_start(GTK_BOX(vbox0), esc, FALSE, FALSE, 5);
     if (allow_shuffle) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(esc), TRUE);
     }
     gtk_signal_connect(GTK_OBJECT(esc), "toggled", (GtkSignalFunc)shuffle_mode_toggle_cb, NULL);
+    gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), esc, _("When a vowel is followed by a (leading) consonant, treat the pair as a consonant followed by a vowel."), NULL);
 
     return vbox0;
 }
@@ -386,18 +336,18 @@ create_appearance_setting(void)
     vbox = gtk_vbox_new(FALSE, 2);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
-    status = gtk_check_button_new_with_label("â ");
+    status = gtk_check_button_new_with_label(_("Make status window"));
     gtk_box_pack_start(GTK_BOX(vbox), status, FALSE, FALSE, 5);
     if (support_status) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(status), TRUE);
     }
     gtk_signal_connect(GTK_OBJECT(status), "toggled", 
     	(GtkSignalFunc)support_status_toggled, NULL);
-    gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), status, "â  ƹ̸ ٽ ž߸ ȿ ֽϴ.", NULL);
+    gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), status, _("To get this change effective, you have to relaunch Ami"), NULL);
 
     hbox = gtk_hbox_new(FALSE, 2);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
-    l = gtk_label_new("ѱۻ¶:");
+    l = gtk_label_new(_("Korean Input Mode Label"));
     gtk_box_pack_start(GTK_BOX(hbox), l, FALSE, FALSE, 0);
     entry = gtk_entry_new();
     gtk_entry_set_text(GTK_ENTRY(entry), hangul_mode_label);
@@ -413,7 +363,7 @@ create_appearance_setting(void)
 
     hbox = gtk_hbox_new(FALSE, 2);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
-    l = gtk_label_new("¶:");
+    l = gtk_label_new(_("English Input Mode Label"));
     gtk_box_pack_start(GTK_BOX(hbox), l, FALSE, FALSE, 0);
     entry = gtk_entry_new();
     gtk_entry_set_text(GTK_ENTRY(entry), english_mode_label);
@@ -427,7 +377,7 @@ create_appearance_setting(void)
     	(GtkSignalFunc)entry_changed, NULL);
     activate_list = g_list_prepend(activate_list, entry);
 
-    under = gtk_check_button_new_with_label("Էâ  ");
+    under = gtk_check_button_new_with_label(_("Use underline during preediting"));
     gtk_box_pack_start(GTK_BOX(vbox), under, FALSE, FALSE, 5);
     if (ami_use_underline) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(under), TRUE);
@@ -435,7 +385,7 @@ create_appearance_setting(void)
     gtk_signal_connect(GTK_OBJECT(under), "toggled", (GtkSignalFunc)underline_mode_toggled, NULL);
     bell_box = gtk_hbox_new(FALSE, 2);
     gtk_box_pack_start(GTK_BOX(vbox), bell_box, FALSE, FALSE, 5);
-    bell_label = gtk_label_new("Ҹ ũ");
+    bell_label = gtk_label_new(_("Bell volume"));
     adj = gtk_adjustment_new (0.0, -100, 110, 1, 10, 10);
     gtk_signal_connect(adj, "value-changed", (GtkSignalFunc)bell_changed, NULL);
     bell_scale = gtk_hscale_new((GtkAdjustment*)adj);
@@ -463,7 +413,7 @@ create_appearance_setting(void)
 	omenu = gtk_option_menu_new();
 	menu = gtk_menu_new();
 	group = NULL;
-	menu_item = gtk_radio_menu_item_new_with_label(group, "⺻");
+	menu_item = gtk_radio_menu_item_new_with_label(group, _("Default"));
 	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", (GtkSignalFunc)pix_changed, NULL);
 	gtk_menu_append(GTK_MENU(menu), menu_item);
 	if (selected ==  -1)
@@ -483,7 +433,7 @@ create_appearance_setting(void)
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
 	gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), selected + 1);
 	hbox = gtk_hbox_new(FALSE, 2);
-	label = gtk_label_new("׸:");
+	label = gtk_label_new(_("Icon"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), omenu, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
@@ -504,7 +454,7 @@ create_key_setting(void)
 
     hbox = gtk_hbox_new(FALSE, 2);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
-    l = gtk_label_new("ѿȯ");
+    l = gtk_label_new(_("Kor-Eng. mode Toggle"));
     gtk_box_pack_start(GTK_BOX(hbox), l, FALSE, FALSE, 0);
     entry = gtk_entry_new();
     key_desc = trigger_keys_to_str(trigger_keys);
@@ -514,14 +464,14 @@ create_key_setting(void)
     gtk_signal_connect(GTK_OBJECT(entry), "activate", 
     	(GtkSignalFunc)activate_cb, NULL);
     gtk_signal_connect(GTK_OBJECT(entry), "update", 
-    	(GtkSignalFunc)key_setting_activate, "ѿ");
+    	(GtkSignalFunc)key_setting_activate, "KorEng");
     gtk_signal_connect(GTK_OBJECT(entry), "changed",
     	(GtkSignalFunc)entry_changed, NULL);
     activate_list = g_list_prepend(activate_list, entry);
 
     hbox = gtk_hbox_new(FALSE, 2);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
-    l = gtk_label_new("ںȯ");
+    l = gtk_label_new(_("Conversion to Hanja"));
     gtk_box_pack_start(GTK_BOX(hbox), l, FALSE, FALSE, 0);
     entry = gtk_entry_new();
     key_desc = trigger_keys_to_str(hanja_keys);
@@ -531,14 +481,14 @@ create_key_setting(void)
     gtk_signal_connect(GTK_OBJECT(entry), "activate", 
     	(GtkSignalFunc)activate_cb, NULL);
     gtk_signal_connect(GTK_OBJECT(entry), "update", 
-    	(GtkSignalFunc)key_setting_activate, "");
+    	(GtkSignalFunc)key_setting_activate, "Hanja");
     gtk_signal_connect(GTK_OBJECT(entry), "changed",
     	(GtkSignalFunc)entry_changed, NULL);
     activate_list = g_list_prepend(activate_list, entry);
 
     hbox = gtk_hbox_new(FALSE, 2);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
-    l = gtk_label_new("ƯԷ");
+    l = gtk_label_new(_("Symbol Input"));
     gtk_box_pack_start(GTK_BOX(hbox), l, FALSE, FALSE, 0);
     entry = gtk_entry_new();
     key_desc = trigger_keys_to_str(special_char_keys);
@@ -548,14 +498,14 @@ create_key_setting(void)
     gtk_signal_connect(GTK_OBJECT(entry), "activate", 
     	(GtkSignalFunc)activate_cb, NULL);
     gtk_signal_connect(GTK_OBJECT(entry), "update", 
-    	(GtkSignalFunc)key_setting_activate, "Ư");
+    	(GtkSignalFunc)key_setting_activate, "SpecialChar");
     gtk_signal_connect(GTK_OBJECT(entry), "changed",
     	(GtkSignalFunc)entry_changed, NULL);
     activate_list = g_list_prepend(activate_list, entry);
 
     hbox = gtk_hbox_new(FALSE, 2);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
-    l = gtk_label_new(" ");
+    l = gtk_label_new(_("Clear Buffer"));
     gtk_box_pack_start(GTK_BOX(hbox), l, FALSE, FALSE, 0);
     entry = gtk_entry_new();
     key_desc = trigger_keys_to_str(flush_keys);
@@ -565,7 +515,7 @@ create_key_setting(void)
     gtk_signal_connect(GTK_OBJECT(entry), "activate", 
     	(GtkSignalFunc)activate_cb, NULL);
     gtk_signal_connect(GTK_OBJECT(entry), "update", 
-    	(GtkSignalFunc)key_setting_activate, " ");
+    	(GtkSignalFunc)key_setting_activate, "BufferClean");
     gtk_signal_connect(GTK_OBJECT(entry), "changed",
     	(GtkSignalFunc)entry_changed, NULL);
     activate_list = g_list_prepend(activate_list, entry);
@@ -582,7 +532,6 @@ create_cp(void)
     GtkWidget *b;
     GtkWidget *sep;
 
-    set_keyboard();
     tooltip = gtk_tooltips_new();
     vbox = gtk_vbox_new(FALSE, 10);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
@@ -592,27 +541,27 @@ create_cp(void)
     	"update", 0, gtk_marshal_NONE__NONE,
 	GTK_TYPE_NONE, 0);
     {
-	/* Ű  */
+	/* set up a keyboard */
 	GtkWidget *keyw, *keylabel;
-	keylabel = gtk_label_new("Ű");
+	keylabel = gtk_label_new(_("KBD Type"));
 	keyw = create_keyboard_setting();
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), keyw, keylabel);
     }
     {
 	GtkWidget *inputw, *inputlabel;
-	inputlabel = gtk_label_new("Է");
+	inputlabel = gtk_label_new(_(_("Input Options")));
 	inputw = create_input_setting();
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), inputw, inputlabel);
     }
     {
 	GtkWidget *keyw, *keylabel;
-	keylabel = gtk_label_new("Ű");
+	keylabel = gtk_label_new(_("Hot Keys"));
 	keyw = create_key_setting();
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), keyw, keylabel);
     }
     {
 	GtkWidget *box, *label;
-	label = gtk_label_new("");
+	label = gtk_label_new(_("Appearance"));
 	box = create_appearance_setting();
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box, label);
     }
@@ -632,20 +581,20 @@ create_cp(void)
 	gtk_box_pack_end(GTK_BOX(h), hbox, FALSE, FALSE, 0);
     }
 
-    b = gtk_button_new_with_label("ƹ̶?");
+    b = gtk_button_new_with_label(_("About.."));
     gtk_container_add(GTK_CONTAINER(hbox), b);
     gtk_signal_connect(GTK_OBJECT(b), "clicked", (GtkSignalFunc)about_ami, NULL);
 
-    b = gtk_button_new_with_label("");
+    b = gtk_button_new_with_label(_("Save Changes"));
     gtk_container_add(GTK_CONTAINER(hbox), b);
     gtk_signal_connect(GTK_OBJECT(b), "clicked", (GtkSignalFunc)save_conf, NULL);
 
     if (ami_run_mode == AMI_NORMAL_RUN) {
-	b = gtk_button_new_with_label("");
+	b = gtk_button_new_with_label(_("Exit"));
 	gtk_container_add(GTK_CONTAINER(hbox), b);
 	gtk_signal_connect(GTK_OBJECT(b), "clicked", (GtkSignalFunc)cp_quit, NULL);
     } else {
-	b = gtk_button_new_with_label("ݱ");
+	b = gtk_button_new_with_label(_("Close"));
 	gtk_container_add(GTK_CONTAINER(hbox), b);
 	gtk_signal_connect(GTK_OBJECT(b), "clicked", (GtkSignalFunc)cp_delete, NULL);
     }
@@ -662,17 +611,17 @@ about_ami(void)
     GString *str;
 
     win = gtk_window_new(GTK_WINDOW_DIALOG);
-    gtk_window_set_title(GTK_WINDOW(win), "ƹ̶?");
+    gtk_window_set_title(GTK_WINDOW(win), _("About Ami"));
     gtk_container_set_border_width(GTK_CONTAINER(win), 5);
     str = g_string_new(NULL);
-    g_string_sprintf(str, "ƹ  %s\n", VERSION);
+    g_string_sprintf(str, _("Ami Version %s\n"), VERSION);
     g_string_append(str, 
-	"\nƹ̴ Ȳġ(hwang@mizi.co.kr) \n"
-	"gtk+ ̿Ͽ  \nGnu General Public License Version 2"
-	" \n Ӱ Ͻ  ִ ѱԷ±Դϴ.\n\n"
-        "gtk+  Ϳ    \n Ѵ.\n"
-	"gtk+  ڼ  \n"
-	"http://www.gtk.org  ֽϴ.");
+	_("\nAmi written by HWANG Chi-deok <hwang@mizi.co.kr> "
+	"built upon gtk+\n is a free Korean X input method server\n"
+	"distributed under GNU General Public License V2. \n\n"
+        "A special thanks goes to Peter who began the gtk+ project\n"
+			  "and Owen who's made it as it is now.\n"
+	"Please, visit <http://www.gtk.org> for more info."));
     label = gtk_label_new(str->str);
     g_string_free(str, TRUE);
     gtk_container_add(GTK_CONTAINER(win), label);
