@@ -7,14 +7,15 @@
 #include "ic.h"
 #include "util.h"
 #include "cli.h"
+#include "utf8.h"
 
-/*  ũ⸦ client ˷ʾ  
-    ũ⸦ Ѵ. ̶ client   ð
-     ũ⸦ ʾ ̸ ˻ϴ 
-   ð  ٽ  ũ⸦ ˾ƳѴ.
-   MIN_WIDTH   ũⰡ   ٽ 
-   ũ⸦ ˾ƺ. netscape  5x5ũ 찡
-   ʱ⿡  ٽ ùٸ ũ ȴ. gtk 1x1 */
+/* If a client does not inform us of the window size, we need to 
+ * figure it out on our own. Sometimes the first attempt gives
+ * us an inappropriate value (smaller than MIN_WIDTH) when it's made
+ * before the window of a client is fully formed. In that case,
+ * we have to give it another try after some delay. Netscape (4.x)
+ * makes a 5 x 5 window at first and then expands(?) it to an
+ * appropriate size. So does gtk with the  initial size of 1 x 1. */
 
 #define MIN_WIDTH 10
 
@@ -95,12 +96,12 @@ dw_set_parent(DispWindow *dw, GdkWindow *parent)
 	if (gdk_window_get_parent(dw->win) == parent) {
 	    return;
 	}
-	if (!dw->ic->top_win && dw->ic->focus_win && 
+	if (!dw->ic->top_win && dw->ic->focus_win &&
 	    dw->ic->focus_win != dw->ic->client_win) {
 	    dw->ic->top_win = dw->ic->client_win;
 	}
 	gdk_window_reparent(dw->win, parent, dw->x, dw->y - dw->font->ascent);
-	/* area ߴٸ ٽ ϵ Ѵ */
+	/* if area was calculated before, reevaluate it ???? */
 	if (dw->area.x < 0) dw->area.width = 1;
 	dw_update_width(dw);
 	return;
@@ -134,7 +135,7 @@ dw_set_parent(DispWindow *dw, GdkWindow *parent)
     dw_set_fg(dw, dw->fg);
     dw_set_bg(dw, dw->bg);
     dw_update_width(dw);
-    if (!dw->ic->top_win && dw->ic->focus_win && 
+    if (!dw->ic->top_win && dw->ic->focus_win &&
     	dw->ic->focus_win != dw->ic->client_win) {
 	dw->ic->top_win = dw->ic->client_win;
     }
@@ -180,13 +181,9 @@ static int
 get_fit_cnum(DispWindow *dw, char *text, int pwidth)
 {
     int count = 0;
-    int mul;
-    while(*text) {
-	if (*text & 0x80) {
-	    mul = 2;
-	} else {
-	    mul = 1;
-	}
+    int mul=MBC_LEN(*text);
+
+    while (*text) {
 	pwidth -= gdk_text_width(dw->font, text, mul);
 	if (pwidth < 0) break;
 	text += mul;
@@ -277,7 +274,7 @@ dw_event_filter(XEvent *xev, GdkEvent *ev, DispWindow *dw)
 	return GDK_FILTER_REMOVE;
     }
     if (xev->xany.type == DestroyNotify) {
-	ic_destroy(dw->ic); 
+	ic_destroy(dw->ic);
 	return GDK_FILTER_REMOVE;
     }
     return GDK_FILTER_CONTINUE;
@@ -329,7 +326,8 @@ dw_get_area_needed(DispWindow *dw, XRectangle *area)
 	if (dw->caret < 0) {
 	    dw->area_needed.width = dw_get_status_needed_width(dw);
 	} else {
-	    dw->area_needed.width = string_width(dw, "");
+	    dw->area_needed.width = string_width(dw, 
+		ami_codeset==AMI_EUC ? "\xb0\xb1" : "\xea\xb0\x80" );
 	}
     }
     if (dw->area_needed.height == 0) {
@@ -361,8 +359,8 @@ dw_set_text(DispWindow *dw, char *text, int caret, int has_temp_hangul)
     dw_update_width(dw);
 }
 
-static int 
-dont_move_to_next_line (DispWindow *dw) 
+static int
+dont_move_to_next_line (DispWindow *dw)
 {
     int y;
 
@@ -385,9 +383,12 @@ dw_set_pos(DispWindow *dw, XPoint *p)
     if (dw->x == p->x && dw->y == p->y) return;
     dw->x = p->x;
     dw->y = p->y;
-    if(debug) g_print("%s: ic=%d x=%d y=%d\n",__FUNCTION__,dw->ic->id, p->x,p->y);
+    if (debug>2)
+	g_print("%s: ic=%d x=%d y=%d\n",__FUNCTION__,dw->ic->id, p->x,p->y);
     if (!dw->win) return;
-    if (dw->x + string_width(dw, "")> dw->area.width) {
+    /* 0xb0 0xa1 : 'GA' in EUC-KR 0xea 0x b0 0x80 : 'GA' in UTF-8 */
+    if (dw->x + string_width(dw, ami_codeset==AMI_EUC ? "\xb0\xa1" : 
+	  "\xea\xb0\x80") > dw->area.width) {
 	if (!dont_move_to_next_line(dw)) {
 	    dw->x = 0;
 	    if (dw->linespace > 0) dw->y += dw->linespace;
@@ -395,7 +396,7 @@ dw_set_pos(DispWindow *dw, XPoint *p)
 	    if (dw->y > dw->area.height - dw->font->descent) {
 		    /* y is too low; jump up */
 		    dw->y  = dw->font->ascent;
-		    
+
 	    }
 	}
     }
@@ -447,7 +448,7 @@ dw_set_font(DispWindow *dw, char *font)
     g_return_if_fail (dw != NULL);
     g_return_if_fail (font != NULL);
 
-    /* qt2.?.x Ŀ   ۲  . Silly! */
+    /* Qt 2.?.x keeps sending the font info. whenever it gets focused. Silly !*/
     if (strcmp(dw->fontname, font) == 0) return;
 
     util_font_unref(dw->font);
@@ -455,7 +456,7 @@ dw_set_font(DispWindow *dw, char *font)
     g_free(dw->fontname);
     dw->fontname = g_strdup(font);
     if(debug) g_print("%s: fontname = %s ic=%d\n", __FUNCTION__, font, dw->ic->id);
-    /* ũ⸦ 츮 ߴٸ ο ۲ÿ ٽ Ѵ. */
+    /* If the size was determined by us, obtain it again for a new font */
     if (dw->area.x < 0) dw->area.width = 0;
     p.x = dw->x;
     p.y = dw->y;
@@ -479,7 +480,7 @@ dw_real_set_bg(DispWindow *dw)
     if (ami_use_underline || dw->caret < 0) {
 	bg = dw->bg;
     } else {
-	/* fg bg 2:8    pixel Ѵ. */
+	/* Mix fg color and bg color with the ratio of 2 : 8 */
 	XColor cfg;
 	XColor cbg;
 	GdkColor newbg;
@@ -578,14 +579,14 @@ dw_draw_simple(DispWindow *dw)
 
     font_height = dw->font->ascent + dw->font->descent;
     if (dw->text[0] != '\0') {
-	gdk_draw_string(dw->win, dw->font, dw->gc, -dw->xoffset, dw->font->ascent, 
+	gdk_draw_string(dw->win, dw->font, dw->gc, -dw->xoffset, dw->font->ascent,
 	    dw->text);
-	if (dw->caret >= 0 && ami_use_underline) 
+	if (dw->caret >= 0 && ami_use_underline)
 	   gdk_draw_line(dw->win, dw->gc, 0, font_height-1, dw->win_width, font_height-1);
     }
     if (dw->has_temp_hangul) {
-	gdk_draw_rectangle(dw->win, dw->gc, 1, 
-		-dw->xoffset + dw->caret_pixel, 0, 
+	gdk_draw_rectangle(dw->win, dw->gc, 1,
+		-dw->xoffset + dw->caret_pixel, 0,
 		dw_seg_width(dw, dw->caret - 2, 2),
 		font_height);
 	gdk_draw_text(dw->win, dw->font, dw->reverse_gc,
@@ -608,28 +609,31 @@ dw_draw_caret(DispWindow *dw)
     font_height = dw->font->ascent + dw->font->descent;
 
     if (dw->has_temp_hangul) {
-	/*ѱ ִ ̸ caretſ ִ ѱ
-	  inversionѼ ׸ */
+	/*In the middle of a Hangul syllable formation, draw it in
+	 * inverse mode instead of the caret. */
+	int len = MBC_LEN_B(dw->text+dw->caret-1);
+	if (debug > 3) g_print("%s: MBC_LEN_B()=%d\n",__FUNCTION__,len);
+	/* FIXME : We may cheat here just using 3 for UTF8 and 2 for 
+	   EUC-KR. J. Shin */
 	if (dw->caret <= dw->cn1) {
-	    /* win1 caret ġ */
+	    /* caret is in win1 */
 	    win = dw->win;
-	    x = dw_seg_width(dw, 0, dw->caret - 2);
+	    x = dw_seg_width(dw, 0, dw->caret - len);
 	} else if (dw->caret <= dw->cn1 + dw->cn2) {
-	    /* win2 caret ġ */
+	    /* caret is in win2 */
 	    win = dw->win2;
-	    x = dw_seg_width(dw, dw->cn1, dw->caret - 2 - dw->cn1);
+	    x = dw_seg_width(dw, dw->cn1, dw->caret - len - dw->cn1);
 	} else {
-	    /* win3 caret ġ */
+	    /* caret is in win3 */
 	    win = dw->win3;
 	    x = dw_seg_width(dw, dw->cn1+dw->cn2,
-	    	             dw->caret - 2 - dw->cn1 - dw->cn2);
+	    	             dw->caret - len - dw->cn1 - dw->cn2);
 	}
 	g_assert (win != NULL);
-	gdk_draw_rectangle(win, dw->gc, 1, x, 0, 
-		dw_seg_width(dw, dw->caret - 2, 2), font_height);
+	gdk_draw_rectangle(win, dw->gc, 1, x, 0,
+		dw_seg_width(dw, dw->caret - len, len), font_height);
 	gdk_draw_text(win, dw->font, dw->reverse_gc,
-		x, dw->font->ascent, dw->text + dw->caret - 2, 2);
-
+		x, dw->font->ascent, dw->text + dw->caret - len, len);
     } else {
 	if (dw->caret <= dw->cn1) {
 	    win = dw->win;
@@ -669,21 +673,21 @@ dw_draw_simple2(DispWindow *dw)
     gdk_window_clear(dw->win);
     gdk_draw_text(dw->win, dw->font, dw->gc, 0, dw->font->ascent, dw->text, dw->cn1);
     if (ami_use_underline)
-	gdk_draw_line(dw->win, dw->gc, 0, height-1, 
+	gdk_draw_line(dw->win, dw->gc, 0, height-1,
 		      dw_seg_width(dw, 0, dw->cn1), height-1);
     if (dw->win2) {
 	gdk_window_clear(dw->win2);
 	gdk_draw_text(dw->win2, dw->font, dw->gc, 0, dw->font->ascent, dw->text + dw->cn1, dw->cn2);
 	if (ami_use_underline)
-		gdk_draw_line(dw->win2, dw->gc, 0, height-1, 
-			      dw_seg_width(dw, dw->cn1, dw->cn2), 
+		gdk_draw_line(dw->win2, dw->gc, 0, height-1,
+			      dw_seg_width(dw, dw->cn1, dw->cn2),
 			      height-1);
     }
     if (dw->win3) {
 	gdk_window_clear(dw->win3);
 	gdk_draw_text(dw->win3, dw->font, dw->gc, 0, dw->font->ascent, dw->text + (dw->cn1+dw->cn2), dw->cn3);
 	if (ami_use_underline)
-		gdk_draw_line(dw->win3, dw->gc, 0, height-1, 
+		gdk_draw_line(dw->win3, dw->gc, 0, height-1,
 			      dw_seg_width(dw, dw->cn1+dw->cn2, dw->cn3),
 			      height-1);
     }
@@ -702,21 +706,21 @@ dw_draw_extra(DispWindow *dw)
     if (dw->cn1)
     	gdk_draw_text(dw->win, dw->font, dw->gc, 0, dw->font->ascent, dw->text, dw->cn1);
     if (ami_use_underline)
-	    gdk_draw_line(dw->win, dw->gc, 0, height-1, 
+	    gdk_draw_line(dw->win, dw->gc, 0, height-1,
 			  dw_seg_width(dw, 0, dw->cn1), height-1);
     if (dw->win2) {
 	gdk_window_clear(dw->win2);
 	gdk_draw_text(dw->win2, dw->font, dw->gc, 0, dw->font->ascent, dw->text + dw->cn1, dw->cn2);
 	if (ami_use_underline)
-		gdk_draw_line(dw->win2, dw->gc, 0, height-1, 
-			      dw_seg_width(dw, dw->cn1, dw->cn2), 
+		gdk_draw_line(dw->win2, dw->gc, 0, height-1,
+			      dw_seg_width(dw, dw->cn1, dw->cn2),
 			      height-1);
     }
     if (dw->win3) {
 	gdk_window_clear(dw->win3);
 	gdk_draw_text(dw->win3, dw->font, dw->gc, 0, dw->font->ascent, dw->text + (dw->cn1+dw->cn2), dw->cn3);
 	if (ami_use_underline)
-		gdk_draw_line(dw->win3, dw->gc, 0, height-1, 
+		gdk_draw_line(dw->win3, dw->gc, 0, height-1,
 			      dw_seg_width(dw, dw->cn1+dw->cn2, dw->cn3),
 			      height-1);
     }
@@ -731,11 +735,11 @@ dw_draw(DispWindow *dw)
     if (dw->win == NULL) return;
     if (!gdk_window_is_visible(dw->win)) return;
 
-    if (dw->caret < 0 || dw->ic->input_style & XIMPreeditArea) 
+    if (dw->caret < 0 || dw->ic->input_style & XIMPreeditArea)
     	dw_draw_simple(dw);
     else if (dont_move_to_next_line(dw))
         dw_draw_simple2(dw);
-    else 
+    else
         dw_draw_extra(dw);
 }
 
@@ -783,7 +787,9 @@ dw_update_width(DispWindow *dw)
 	    dw->xoffset = 0;
 	}
 	if (dw->has_temp_hangul) {
-	    dw->caret_pixel = caret_pixel - dw_seg_width(dw, dw->caret - 2, 2);
+		  int len= MBC_LEN_B(dw->text + dw->caret-1);
+	    dw->caret_pixel = (caret_pixel
+			       - dw_seg_width(dw, dw->caret - len, len));
 	} else {
 	    dw->caret_pixel = caret_pixel;
 	}
@@ -791,7 +797,6 @@ dw_update_width(DispWindow *dw)
 	return;
     }
 
-
     /* get_cn1 */
     max_x = dw->area.x < 0?dw->area.width:(dw->area.x+dw->area.width);
     if (dw->x + tw <= max_x) {
@@ -800,14 +805,14 @@ dw_update_width(DispWindow *dw)
 	if (cn1 == dw->caret && !dw->has_temp_hangul) w1 += 2;
 	cn2 = 0;
 	if (dw->win2) {
-	    gdk_window_remove_filter (dw->win2, 
+	    gdk_window_remove_filter (dw->win2,
 	    	(GdkFilterFunc)dw_event_filter_extra, dw);
 	    gdk_window_destroy(dw->win2);
 	    dw->win2 = NULL;
 	}
 	cn3 = 0;
 	if (dw->win3) {
-	    gdk_window_remove_filter (dw->win3, 
+	    gdk_window_remove_filter (dw->win3,
 	    	(GdkFilterFunc)dw_event_filter_extra, dw);
 	    gdk_window_destroy(dw->win3);
 	    dw->win3 = NULL;
@@ -818,14 +823,14 @@ dw_update_width(DispWindow *dw)
 	if (cn1 == dw->caret && !dw->has_temp_hangul) w1 += 2;
 	cn2 = 0;
 	if (dw->win2) {
-	    gdk_window_remove_filter (dw->win2, 
+	    gdk_window_remove_filter (dw->win2,
 	    	(GdkFilterFunc)dw_event_filter_extra, dw);
 	    gdk_window_destroy(dw->win2);
 	    dw->win2 = NULL;
 	}
 	cn3 = 0;
 	if (dw->win3) {
-	    gdk_window_remove_filter (dw->win3, 
+	    gdk_window_remove_filter (dw->win3,
 	    	(GdkFilterFunc)dw_event_filter_extra, dw);
 	    gdk_window_destroy(dw->win3);
 	    dw->win3 = NULL;
@@ -847,7 +852,7 @@ dw_update_width(DispWindow *dw)
 		gdk_window_move_resize(dw->win2, 0, y2-dw->font->ascent, w2, font_height);
 	    }
 	    cn3 = text_len - cn1 - cn2;
-	    if (dw->caret == text_len && !dw->has_temp_hangul) 
+	    if (dw->caret == text_len && !dw->has_temp_hangul)
 	    	w3 = string_width(dw, dw->text + (cn1+cn2)) + 2;
 	    else
 	    	w3 = string_width(dw, dw->text + (cn1+cn2));
@@ -862,7 +867,7 @@ dw_update_width(DispWindow *dw)
 	    }
 	} else {
 	    cn2 = text_len - cn1;
-	    if (dw->caret == text_len && !dw->has_temp_hangul) 
+	    if (dw->caret == text_len && !dw->has_temp_hangul)
 	    	w2 = string_width(dw, dw->text + cn1) + 2;
 	    else
 	    	w2 = string_width(dw, dw->text + cn1);
@@ -877,7 +882,7 @@ dw_update_width(DispWindow *dw)
 	    }
 	    cn3 = 0;
 	    if (dw->win3) {
-		gdk_window_remove_filter (dw->win3, 
+		gdk_window_remove_filter (dw->win3,
 		    (GdkFilterFunc)dw_event_filter_extra, dw);
 		gdk_window_destroy(dw->win3);
 		dw->win3 = NULL;
