@@ -8,7 +8,10 @@
 #include "cb.h"
 #include "util.h"
 #include "hinput.h"
+#include "comp.h"
 #include <gdk/gdkkeysyms.h>
+#include <stdio.h>
+#include "utf8.h"
 
 char *
 editing_mbstocts(char *s)
@@ -17,6 +20,16 @@ editing_mbstocts(char *s)
 
     XmbTextListToTextProperty(gdk_display, &s, 1,
 			      XCompoundTextStyle, &tp);
+    if (debug) {
+	int i;
+	fprintf(stderr,"%s: mbs= ", __FUNCTION__);
+	for (i=0; i< strlen(s); i++) 
+	    fprintf(stderr,"%02x%c", (unsigned char) s[i], i!=strlen(s)-1 ? ' ' : '\n');
+	fprintf(stderr,"%s: cts= ", __FUNCTION__);
+	for (i=0; i< strlen(tp.value); i++) 
+	    fprintf(stderr,"%02x%c", tp.value[i], 
+		i!=strlen(tp.value)-1 ? ' ' : '\n');
+    }
     return tp.value;
 }
 
@@ -75,7 +88,7 @@ editing_toggle(IC *ic)
     ic->composing_hangul = ic->han->composing_hangul;
 
     editing_flush(ic);
-    ami_hangul_input_clear();
+    composer_reset(ic->han->composer);
     if (ic->edit_win) {
 	if (ic->han->composing_hangul) {
 	    dw_show(ic->edit_win);
@@ -103,7 +116,6 @@ editing_flush(IC *ic)
     editing_commit(ic, ic->han->buf);
     editing_send_caret_position(ic);
     editing_reset(ic);
-    //editing_commit(ic, s);
     g_free(s);
     return 1;
 }
@@ -122,7 +134,6 @@ editing_flush_without_reset(IC *ic)
     }
     ic->han->buf[0] = '\0';
     ic->han->pos = ic->han->len = 0;
-    ic->han->has_temp_hangul = 0;
     if (ic->edit_win) {
 	dw_set_text(ic->edit_win, "", 0, 0);
     }
@@ -130,19 +141,24 @@ editing_flush_without_reset(IC *ic)
 }
 
 int
-editing_flush_with_c(IC *ic, char c)
+editing_flush_with_str(IC *ic, char *s)
 {
+    char * buf;
+    int pos;
+    int add_len = strlen(s);
+
     g_return_val_if_fail (ic != NULL, 0);
     g_return_val_if_fail (ic->han != NULL, 0);
     if (ic->han->len == 0) {
-	char buf[2]; 
-	buf[0] = c; buf[1] = 0;
-	editing_commit(ic, buf);
+	editing_commit(ic, s);
 	return 1;
     }
-    memmove(ic->han->buf + ic->han->pos + 1, ic->han->buf + ic->han->pos, ic->han->len - ic->han->pos + 1);
-    ic->han->buf[ic->han->pos] = c; 
-    editing_commit(ic, ic->han->buf);
+
+    buf = ic->han->buf;
+    pos = ic->han->pos;
+    memmove(buf + pos + add_len, buf + pos, ic->han->len - pos + 1);
+    memcpy(buf + pos, s, add_len);
+    editing_commit(ic, buf);
     ic->han->pos++;
     editing_send_caret_position(ic);
     ic->han->pos--;
@@ -159,10 +175,9 @@ editing_reset(IC *ic)
     if (ic->input_style & XIMPreeditCallbacks) {
 	cb_edit_reset(ic);
     }
-    ami_hangul_input_clear();
     ic->han->buf[0] = '\0';
     ic->han->pos = ic->han->len = 0;
-    ic->han->has_temp_hangul = 0;
+    composer_reset(ic->han->composer);
     if (ic->edit_win) {
 	dw_set_text(ic->edit_win, "", 0, 0);
     }
@@ -175,29 +190,50 @@ editing_correct_insert(IC *ic, guchar *buf)
 
     int pos;
     guchar *hbuf;
+    int len1=2;  /* the length of a replacement char. */
+    int len2=2;  /* the length of a char. to insert */
+    int len0=2;  /* the length of a char. to be replaced */
+    int lenchg;  /* net change in the length of ic->han->buf */
     g_return_if_fail (ic != NULL);
     g_return_if_fail (ic->han != NULL);
 
     pos = ic->han->pos;
 
-    buf[4] = '\0';
+    /* By inspecting the first byte, we can tell how many octets
+     * are taken by a UTF-8 character. If it's in [0xe0,0xef],
+     * it should be 3-octet-long */
+    if ( ami_codeset==AMI_UTF8 && IS_UTF8_3BYTE(buf[0])) 
+      len1=3;
+    if ( ami_codeset==AMI_UTF8 && IS_UTF8_3BYTE(buf[len1]))
+      len2=3;
+
+    buf[len1+len2] = '\0';
 
     if (ic->input_style & XIMPreeditCallbacks) {
 	cb_edit_correct_insert(ic, buf);
     }
 
-    ami_hangul_check_bufsize(ic->han, 2);
-
     hbuf = ic->han->buf;
+    /* if the second byte from the end is still in the range [0x80,0xbf],
+     * it should be at least 3-octet-long. It could be 4 octets long,
+     * but at the moment we don't care about characters beyond BMP. */
+    if ( ami_codeset==AMI_UTF8 && IS_UTF8_TRAIL(hbuf[pos-2]) )
+      len0=3;
+
+    lenchg = len2+len1-len0;
+    ami_hangul_check_bufsize(ic->han, lenchg);
+
+
     if (ic->han->len > pos)
-        memmove(hbuf + pos + 2, hbuf + pos, ic->han->len - pos);
-    memcpy(hbuf + pos - 2, buf, 4);
-    ic->han->pos += 2;
-    ic->han->len += 2;
+        memmove(hbuf + pos + lenchg, hbuf + pos, ic->han->len - pos);
+    memcpy(hbuf + pos - len0, buf, len1 + len2);
+    ic->han->pos += lenchg;
+    ic->han->len += lenchg;
     hbuf[ic->han->len] = '\0';
     if (ic->edit_win) {
 	editing_edit_win_update(ic);
-    } 
+    }
+
 }
 
 int
@@ -206,12 +242,15 @@ editing_forward_delete(IC *ic)
     int pos;
     guchar *hbuf;
     int howmuch;
+
     g_return_val_if_fail (ic != NULL, 0);
     g_return_val_if_fail (ic->han != NULL, 0);
 
     pos = ic->han->pos;
     hbuf = ic->han->buf;
-    howmuch = hbuf[pos] & 0x80 ? 2:1;
+    howmuch = MBC_LEN(hbuf[pos]);
+
+    g_assert(howmuch);
 
     if (pos + howmuch > ic->han->len) howmuch = ic->han->len - pos;
     if (howmuch <= 0) return FALSE;
@@ -243,8 +282,22 @@ editing_backward_delete(IC *ic)
 	cb_edit_backward_delete(ic);
     }
 
+    howmuch = MBC_LEN_B(hbuf+pos-1); 
+#if 0
+    if ( ami_codeset == AMI_EUC) 
+        howmuch = hbuf[pos-1] & 0x80 ? 2:1;
+    else
+				howmuch = ! (hbuf[pos-1] & 0x80) ? 1 : 
+	          IS_UTF8_2BYTE(hbuf[pos-2])  ? 2  : 
+	          IS_UTF8_3BYTE(hbuf[pos-3])  ? 3  : 0;
+#endif
+    if (debug) 
+      fprintf(stderr,"%s: *(hbuf+pos-3) = %02x %02x %02x\n"
+      "pos=%d howmuch=%d ic->han->len=%d\n",
+  	  __FUNCTION__,hbuf[pos-3],hbuf[pos-2],hbuf[pos-1],
+   	  pos,howmuch,ic->han->len);
 
-    howmuch = hbuf[pos - 1] & 0x80 ? 2:1;
+    g_assert(howmuch);
     if (pos - howmuch < 0) howmuch = pos;
     memmove(hbuf + pos - howmuch, hbuf + pos, ic->han->len - pos);
     ic->han->len -= howmuch;
@@ -252,7 +305,7 @@ editing_backward_delete(IC *ic)
     hbuf[ic->han->len] = '\0';
     if (ic->edit_win) {
 	editing_edit_win_update(ic);
-    } 
+    }
     return TRUE;
 }
 
@@ -287,25 +340,31 @@ editing_insert(IC *ic, guchar *buf, int size)
     ic->han->pos = pos + size;
     if (ic->edit_win) {
 	editing_edit_win_update(ic);
-    } 
+    }
 }
 
 void
 editing_correct(IC *ic, guchar *buf)
 {
     int pos;
+    int len=2;
+
     g_return_if_fail (ic != NULL);
     g_return_if_fail (ic->han != NULL);
+
     pos = ic->han->pos;
-    buf[2] = '\0';
+    if ( ami_codeset==AMI_UTF8 && IS_UTF8_3BYTE(buf[0]) )
+	len=3;
+
+    buf[len] = '\0';
+
     if (ic->input_style & XIMPreeditCallbacks) {
     	cb_edit_correct(ic, buf);
     }
-    ic->han->buf[pos - 2] = buf[0];
-    ic->han->buf[pos - 1] = buf[1];
+    memcpy(ic->han->buf+(pos-len),buf,len);
     if (ic->edit_win) {
 	editing_edit_win_update(ic);
-    } 
+    }
 }
 
 void
@@ -318,7 +377,7 @@ editing_hanja_replace(IC *ic, guchar *hanja)
     GString *str = g_string_new(NULL);
     char *s = hanja_formats[ami_hanja_subst_mode-1];
     while (*s) {
-	if (strncmp(s, "", 4) == 0) {
+	if (strncmp(s, "ѱ", 4) == 0) {
 	    s += 4;
 	    g_string_append(str, hangul);
 	} else if (strncmp(s, "", 4) == 0) {
@@ -340,13 +399,13 @@ editing_hanja_replace(IC *ic, guchar *hanja)
     ic->han->pos += str->len - len;
     if (ic->edit_win) {
 	editing_edit_win_update(ic);
-    } 
+    }
     g_free(hangul);
     g_string_free(str, TRUE);
     if (debug) g_print("%s\n", buf);
 }
 
-/* ̵ ϸ TRUE, ϸ FALSE */ 
+/* return TRUE if successful in moving. otherwise, return FALSE */
 int
 editing_move(IC *ic, int howmuch)
 {
@@ -361,6 +420,8 @@ editing_move(IC *ic, int howmuch)
     if (howmuch > 0) {
 	while (howmuch--) {
 	    if (buf[pos] & 0x80) pos++;
+            if ( ami_codeset==AMI_UTF8 && ! ((0xe0 ^ buf[pos]) & 0x10) )
+		pos++;
 	    if (++pos >= len) {
 		pos = len;
 		break;
@@ -373,6 +434,8 @@ editing_move(IC *ic, int howmuch)
 		break;
 	    }
 	    if (buf[pos] & 0x80) pos--;
+	    if ( ami_codeset==AMI_UTF8 && !((0x80 ^ buf[pos]) & 0x40) )
+	      pos--;
 	}
     }
     if (ic->han->pos == pos) return FALSE;
@@ -391,7 +454,7 @@ editing_edit_win_update(IC *ic)
     g_return_if_fail (ic != NULL);
     g_return_if_fail (ic->han != NULL);
     if (!ic->edit_win) return;
-    dw_set_text(ic->edit_win, ic->han->buf, ic->han->pos, ic->han->has_temp_hangul);
+    dw_set_text(ic->edit_win, ic->han->buf, ic->han->pos, composer_has_temp_hangul(ic->han->composer));
 }
 
 void
@@ -413,10 +476,9 @@ editing_clear_temp(IC *ic)
 {
     g_return_if_fail (ic != NULL);
     g_return_if_fail (ic->han != NULL);
-    if (!ic->han->has_temp_hangul) return;
+    if (!composer_has_temp_hangul(ic->han->composer)) return;
     if (ic->input_style & XIMPreeditCallbacks) cb_edit_clear_temp(ic);
-    ic->han->has_temp_hangul = 0;
-    ami_hangul_input_clear();
+    composer_reset(ic->han->composer);
 }
 
 void
@@ -427,7 +489,8 @@ editing_send_caret_position(IC *ic)
     int howmuch;
     g_return_if_fail (ic != NULL);
     g_return_if_fail (ic->han != NULL);
-    if (ic->han->len == ic->han->pos) return;
+    /* XXX: assert(len >= pos) */
+    if (ic->han->len <= ic->han->pos) return;
     howmuch = util_get_mb_strlen(ic->han->buf + ic->han->pos, -1);
     if (debug) g_print("%s: %d\n",__FUNCTION__, howmuch);
 
