@@ -15,7 +15,7 @@ void editing_correct_insert(IC *ic, guchar *buf);
 void editing_hanja_replace(IC *ic, guchar *buf);
 int editing_flush(IC *ic);
 int editing_flush_without_reset(IC *ic);
-int editing_flush_with_c(IC *ic, char c);
+int editing_flush_with_str(IC *ic, char *s);
 void editing_clear_temp(IC *ic);
 void editing_status_win_update(IC *ic);
 void editing_edit_win_update(IC *ic);
