@@ -1,4 +1,5 @@
 /* this file is a part of Ami software, (C) Hwang chi-deok 1999 */
+
 #include "ic.h"
 #include "ami.h"
 #include "applet.h"
@@ -12,7 +13,8 @@
 #include "hanja.h"
 #include "util.h"
 #include <gdk/gdkkeysyms.h>
-
+#include "comp.h"
+#include "utf8.h"
 
 IC *current_focus_ic = NULL;
 
@@ -182,11 +184,9 @@ real_event_send(IMForwardEventStruct *call_data)
 }
 
 /*
-commit ForwardEvent Բ  client event ޴ 
-ڹٲ 찡 .
-̸ ϱ ForwardEvent   ð  
-Ʈ Ѵ.
-idle_add   ȵȴ.
+ * When commit and ForwardEvent are sent together, some clients get them
+ * in the reverse order. To avoid that, we send ForwardEvent after
+ * some delay. idle_add wouldn't help here. 
 */
 static int
 send_event_later(IMForwardEventStruct *call_data)
@@ -197,24 +197,6 @@ send_event_later(IMForwardEventStruct *call_data)
     return 0;
 }
 
-static char dvorak_map[] = 
-    /* !"#$%&'()*+,-./ */
-      "!Q#$%&q()*}w'e["
-
-    /* 0123456789:;<=>? */
-      "0123456789ZzW]E{" 
-
-    /* @ABCDEFGHIJKLMNO */
-      "@ANIHDYUJGCVPMLS"
-
-    /* PQRSTUVWXYZ[ \]^ _ */
-      "RXO:KF><BT?-\\=^\""
-
-    /* `abcdefghijklmno */
-      "`anihdyujgcvpmls"
-
-    /* pqrstuvwxyz{|}~ */
-      "rxo;kf.,bt/_|+~";
 
 int
 handler_process_key(XIMS ims, IMForwardEventStruct *call_data)
@@ -226,12 +208,12 @@ handler_process_key(XIMS ims, IMForwardEventStruct *call_data)
     int hot_key;
     IC *ic;
     guchar c;
-    char inputbuf[256];
+    Composer *composer;
 
     ic = ic_find(call_data);
     if (!ic->composing_hangul || !ic->han) {
-	/*  ̰ų ƴϸ  focusing  ʴ ic 
-	   event  */
+	/* If it's in English mode or it doesn't have the focus, 
+	 * return the event. */
 	IMForwardEvent(ims, (gpointer)call_data);
 	return 0;
     };
@@ -253,7 +235,7 @@ handler_process_key(XIMS ims, IMForwardEventStruct *call_data)
     memset(strbuf, 0, STRBUFLEN);
     kev = (XKeyEvent*)&call_data->event;
 
-    if (kev->state & LockMask) kev->state &= ~LockMask; 
+    if (kev->state & LockMask) kev->state &= ~LockMask;
 
     count = XLookupString(kev, strbuf, STRBUFLEN, &keysym, NULL);
     strbuf[count] = 0;
@@ -264,38 +246,36 @@ handler_process_key(XIMS ims, IMForwardEventStruct *call_data)
     }
 
     hot_key = check_hotkey(ic, kev->state, keysym);
-    ami_hangul_ic_change(ic);
-    if (hot_key == HOTKEY_DONE) 
+    composer = ic->han->composer;
+
+    if (hot_key == HOTKEY_DONE)
 	return 1;
     c = strbuf[0];
     if (keysym == GDK_BackSpace) c = '\b';
-    if (c != '\b' && 
+    if (c != '\b' &&
         (hot_key == HOTKEY_FORWARD || (count == 0) ||
-	    (keysym == GDK_space && ic->han->pos == ic->han->len) || 
+	    (keysym == GDK_space && ic->han->pos == ic->han->len) ||
 	    (count == 1 && (c < '!' || c > '~')))) {
-	ic->han->has_temp_hangul = 0;
-	if (ic->han->len > 0) {
+	int l;
+
+	composer_reset(composer);
+	l = ic->han->len;
+	if (l > 0) {
 	    editing_flush(ic);
-	    send_event_later(call_data);
-	} else {
-	    IMForwardEvent(ims, (gpointer)call_data);
 	}
 	if (esc_han_toggle && keysym == GDK_Escape) {
 	    editing_toggle(ic);
 	    draw_applet_image(AMI_PIXMAP_ENGLISH);
 	    IMPreeditEnd(ims, (gpointer)call_data);
 	}
+	if (l > 0)
+	    send_event_later(call_data);
+	else
+	    IMForwardEvent(ims, (gpointer)call_data);
 	return 1;
     }
-    if (ami_ekeyboard == 2) {
-	/* dvorak type english keyboard */
-	if (c >= '!' && c < ('!' + sizeof(dvorak_map)/sizeof(dvorak_map[0])))
-	    strbuf[0] = dvorak_map[strbuf[0] - '!'];
-    }
-    switch (ami_hangul_input_process (inputbuf, strbuf)) {
-	case AUTOMATA_NULL:
-	case AUTOMATA_SPECIAL:
-	    ic->han->has_temp_hangul = 0;
+    switch (composer_process (composer, strbuf[0])) {
+	case COMPOSER_RAW:
 	    if (c == '\b') {
 		if (ic->han->pos > 0) {
 		    editing_backward_delete(ic);
@@ -304,39 +284,53 @@ handler_process_key(XIMS ims, IMForwardEventStruct *call_data)
 		}
 		return 1;
 	    }
-	    editing_flush_with_c(ic, strbuf[0]);
-	    return 1;
+	    editing_flush_with_str(ic, strbuf);
 	    break;
-	case AUTOMATA_CORRECT:
-	    editing_correct(ic, inputbuf);
+	case COMPOSER_NULL:
+	    editing_backward_delete(ic);
 	    break;
-	case AUTOMATA_NEW:
-	    ic->han->has_temp_hangul = 1;
-	    if (ami_editing_mode == AMI_EDITING_CHAR_MODE &&
-	        ic->han->pos > 0) {
+	case COMPOSER_NOT_HAN:
+	    editing_flush_with_str(ic, composer_get_buffer(composer));
+	    break;
+	case COMPOSER_CORRECT:
+	    editing_correct(ic, composer_get_buffer(composer));
+	    break;
+	case COMPOSER_NEW:
+	{
+	    char *inputbuf = composer_get_buffer(composer);
+	    int len=2;
+            if ( ami_codeset==AMI_UTF8 && IS_UTF8_3BYTE(inputbuf[0])) 
+		len=3;
+	    if (ami_editing_mode == AMI_EDITING_CHAR_MODE
+		&& ic->han->pos > 0) {
 		editing_flush_without_reset(ic);
-		editing_insert(ic, inputbuf, 2);
+		editing_insert(ic, inputbuf, len);
 		return 1;
 	    }
-	    editing_insert(ic, inputbuf, 2);
+	    editing_insert(ic, inputbuf, len);
 	    break;
-	case AUTOMATA_CORRECT_NEW:
+	}
+	case COMPOSER_CORRECT_NEW:
 	    if (ami_editing_mode == AMI_EDITING_CHAR_MODE) {
-		char buf[3];
-		buf[0] = inputbuf[0];
-		buf[1] = inputbuf[1];
-		buf[2] = '\0';
-		editing_commit(ic, buf);
-		editing_correct(ic, inputbuf + 2);
+		char buf[5];
+		char *inputbuf = composer_get_buffer(composer);
+		int len=MBC_LEN(inputbuf[0]);
+		strncpy(buf,inputbuf,len);
+ 	        buf[len] = '\0';
+
+		editing_correct(ic, buf);
+		editing_flush_without_reset(ic);
+		editing_insert(ic, inputbuf + len, len);
 		return 1;
 	    }
-	    editing_correct_insert(ic, inputbuf);
+	    editing_correct_insert(ic, composer_get_buffer(composer));
 	    break;
-	case AUTOMATA_ERROR:
+	case COMPOSER_ERROR:
 	    XBell(gdk_display, (bell_intensity>-80)?bell_intensity-20:-100);
 	    break;
-	
-      break;
+
+	default:
+	    break;
     }
     return TRUE;
 }
@@ -387,7 +381,7 @@ handler_trigger_notify(XIMS ims, IMTriggerNotifyStruct *call_data)
 
     draw_applet_image(AMI_PIXMAP_HANGUL);
     if (call_data->flag == 0) {	/* on key */
-	/* Here, the start of preediting is notified from IMlibrary, which 
+	/* Here, the start of preediting is notified from IMlibrary, which
 	   is the only way to start preediting in case of Dynamic Event
 	   Flow, because ON key is mandatary for Dynamic Event Flow. */
 	ic = ic_find(call_data);
@@ -453,7 +447,6 @@ handler_ic_focus_unset(XIMS ims, IMChangeFocusStruct *call_data)
 	dw_unset_focus(ic->status_win);
 	cli_unset_status_ic(ic);
     }
-    if (ic->han != NULL) ami_hangul_state_get(ic->han);
     current_focus_ic = NULL;
     draw_applet_image(AMI_PIXMAP_NONE);
     return TRUE;
@@ -466,10 +459,10 @@ handler_ic_focus_set(XIMS ims, IMChangeFocusStruct *call_data)
     ic = ic_find(call_data);
 
     if (ic_is_valid(current_focus_ic)) {
-	/* unset focus ҷ ¿ focus set ҷ  */
-	/* window  â̿ focusing ̵ɶ Focus event 
-	   ڹٲ 찡 . */
-	ami_hangul_state_get(current_focus_ic->han);
+	/* If focus set is invoked without unset focus called upon,
+	 * the order of Focus events can be reversed when the
+	 * focusing moves from one window to the other (???)
+	 */
 	if (current_focus_ic->edit_win) {
 	    dw_unset_focus(current_focus_ic->edit_win);
 	}
@@ -482,25 +475,31 @@ handler_ic_focus_set(XIMS ims, IMChangeFocusStruct *call_data)
 
     if (ic->top_win == NULL) {
 	if (!ic->client_win) ic->client_win = GDK_ROOT_PARENT();
-	/* X window󿡼 top level window Ѵ. */
-	/* netscapeͰ client window focus window ٴ  
-	   client window ޵Ǿٰ Ѵ. */
+	/* Obtain top level window on X window. */
+	/* In case client window and focus window are different as in Netscape
+	 * 4.x(?), assume the correct client window info. is carried over.
+	 */
 	if (ic->focus_win && ic->client_win != ic->focus_win) {
 	    ic->top_win = ic->client_win;
 	} else if (ic->client_win == GDK_ROOT_PARENT()) {
 	    ic->top_win = GDK_ROOT_PARENT();
 	} else {
-	    /* gtk+ entry text ׻ client window focus window
-	       . ̶ 츮  ¥ client window ãƳ.*/
+	    /* For gtk+ entry and text widgets, client window is always
+	     * the same as focus window so that we need to get it for 
+	     * ourselves.*/
 	    ic->top_win = util_get_top_window (ic->client_win);
+	    if (ic->top_win == NULL)
+	    {
+		if (debug) g_print("Cannot get ic top_win\n");
+		ic->top_win = ic->client_win;
+	    }
 	}
     }
 
     cli_set_edit_ic(ic);
-    ami_hangul_state_put(ic->han);
     if (debug) g_print("%s: ic=%d han_len=%d han=%p "
-                       "han->buf=%p han->buf=%s\n", __FUNCTION__, 
-		       ic->id, ic->han->len, ic->han, 
+                       "han->buf=%p han->buf=%s\n", __FUNCTION__,
+		       ic->id, ic->han->len, ic->han,
 		       ic->han->buf, ic->han->buf);
     if (ic->composing_hangul != ic->han->composing_hangul) {
 	if (ic->han->composing_hangul) {
@@ -519,7 +518,6 @@ handler_ic_focus_set(XIMS ims, IMChangeFocusStruct *call_data)
 	    draw_applet_image(AMI_PIXMAP_ENGLISH);
     }
     cli_set_status_ic(ic);
-    //editing_commit(ic, "");
     current_focus_ic = ic;
     if (ic->han->aux_input) {
 	gtk_object_set_data((GtkObject *)ic->han->aux_input, "ic", ic);
@@ -547,15 +545,15 @@ handler_ic_reset(XIMS ims, IMResetICStruct *call_data)
     ic = ic_find(call_data);
     g_return_val_if_fail (ic != NULL, FALSE);
     if (debug) g_print("%s ic=%d\n",__FUNCTION__, ic->id);
-    ami_hangul_input_clear();
     if (ic->han == NULL || ic->han->buf == NULL) {
 	commit_string = g_strdup(""); //editing_mbstocts(ic->han->buf);
 	call_data->commit_string = commit_string;
 	call_data->length = strlen(commit_string);
-        if (ic->input_style & XIMPreeditCallbacks) 
+        if (ic->input_style & XIMPreeditCallbacks)
 	    cb_cleanup = do_cb_cleanup;
 	return TRUE;
     }
+    composer_reset(ic->han->composer);
     commit_string = editing_mbstocts(ic->han->buf);
     call_data->commit_string = commit_string;
     call_data->length = strlen(commit_string);
