@@ -11,6 +11,8 @@
 #include "edit.h"
 #include "chartable.h"
 #include "ami.h"
+#include "util.h"
+#include "utf8.h"
 
 typedef struct {
     off_t filesize;
@@ -19,7 +21,7 @@ typedef struct {
     int ref;
 } HanjaDic;
 
-HanjaDic *hanjadic = NULL;
+static HanjaDic *hanjadic = NULL;
 
 static void hanja_dialog_destroyed(GtkWidget *widget);
 static void hanja_dict_load(void);
@@ -41,7 +43,7 @@ hanja_find_candidates(guchar *hanbuf, int current_pos, int *num)
     strncpy(search, hanbuf, current_pos);
     search[current_pos] = ' ';
     search[current_pos+1] = '\0';
-    for (i=0;i<current_pos;i += 2) {
+    for (i = 0; i < current_pos; i += MBC_LEN(search[i])) {
 	char *s = hanja_find_word(search + i);
 	if (s) {
 	    char *last = strchr(s, '\n');
@@ -56,7 +58,9 @@ hanja_find_candidates(guchar *hanbuf, int current_pos, int *num)
 		s = strtok(NULL, " ");
 		if (s) buf[(*num)++] = g_strdup(s);
 	    }
+	    g_free(s2);
 	}
+	    g_free(s);
     }
     g_free(search);
     if (*num == 0) return NULL;
@@ -81,9 +85,10 @@ hanja_get_total_len(guchar **words, int max_col)
 {
     int total_len = 0;
     int col = 0;
-    while(*words) {
+
+    while (*words) {
 	int len;
-	len = strlen(*words)/2;
+	len = util_get_mb_strlen(*words,-1);
 	if (col + len > max_col) {
 	    total_len += max_col - col;
 	    col = len;
@@ -101,10 +106,12 @@ static void
 hanja_subst_mode_toggle_cb(GtkWidget *w, gpointer data)
 {
     if (GTK_TOGGLE_BUTTON(w)->active) {
-	CharTable *table;
 	ami_hanja_subst_mode = GPOINTER_TO_INT(data);
+#if 0
+	CharTable *table;
 	table = gtk_object_get_data((GtkObject *)gtk_widget_get_toplevel(w), "ctable");
 	char_table_selected(table);
+#endif
     }
 }
 
@@ -117,6 +124,7 @@ hanja_dialog(HangulState *han)
     guchar **table_buf;
     guchar **s;
     gchar **hanja_formats;
+    GString *fmts_xlat;
     int col, max_col, total_len, len, max_len;
     int i, k, num;
     GSList *group;
@@ -126,7 +134,7 @@ hanja_dialog(HangulState *han)
     candidates = hanja_find_candidates(han->buf, han->pos, &num);
     if (candidates == NULL) return NULL;
 
-    max_len = strlen(candidates[0])/2;
+    max_len = util_get_mb_strlen(candidates[0],-1);
 
     if (num < 20) max_col = MAX(max_len, 5);
     else max_col = MAX(max_len, 10);
@@ -143,8 +151,8 @@ hanja_dialog(HangulState *han)
     col = 0;
     k = 0;
     s = candidates;
-    while(*s) {
-	len = strlen(*s)/2;
+    while (*s) {
+	len = util_get_mb_strlen(*s,-1);
 	if (col + len > max_col) {
 	    while(col++<max_col) table_buf[k++] = NULL;
 	    col = len;
@@ -161,10 +169,10 @@ hanja_dialog(HangulState *han)
 
     win = gtk_window_new(GTK_WINDOW_DIALOG);
     gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER);
-    gtk_window_set_title(GTK_WINDOW(win), "ƹ:  Է");
+    gtk_window_set_title(GTK_WINDOW(win), _("Ami: Hanja Input"));
     gtk_signal_connect((GtkObject *)win, "key_press_event", (GtkSignalFunc)key_press_cb, NULL);
     gtk_container_set_border_width(GTK_CONTAINER(win), 10);
-    gtk_signal_connect(GTK_OBJECT(win), "destroy", 
+    gtk_signal_connect(GTK_OBJECT(win), "destroy",
     	(GtkSignalFunc)hanja_dialog_destroyed, NULL);
 
     vbox = gtk_vbox_new(FALSE, 2);
@@ -177,7 +185,7 @@ hanja_dialog(HangulState *han)
     ctable = char_table_new((char **)table_buf, max_col, total_len);
     gtk_object_set_data(GTK_OBJECT(win), "ctable", ctable);
     gtk_container_add((GtkContainer *)frame, ctable);
-    gtk_signal_connect(GTK_OBJECT(ctable), "selected", 
+    gtk_signal_connect(GTK_OBJECT(ctable), "selected",
     	(GtkSignalFunc)hanja_dialog_selected, NULL);
 
     for(i=0;i<num;i++) g_free(candidates[i]);
@@ -186,7 +194,8 @@ hanja_dialog(HangulState *han)
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
-    hanja_formats = g_strsplit(ami_hanja_subst_format, ":", -1);
+    fmts_xlat = util_xlate_hanja_fmts();
+    hanja_formats = g_strsplit(fmts_xlat->str, ":", -1);
     group = NULL;
     for(i=0;hanja_formats[i] != NULL; i++) {
 	b = gtk_radio_button_new_with_label(group, hanja_formats[i]);
@@ -194,11 +203,12 @@ hanja_dialog(HangulState *han)
 	gtk_container_add(GTK_CONTAINER(hbox), b);
 	if (ami_hanja_subst_mode == (i+1))
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), TRUE);
-	gtk_signal_connect(GTK_OBJECT(b), "toggled", 
-		(GtkSignalFunc)hanja_subst_mode_toggle_cb, 
+	gtk_signal_connect(GTK_OBJECT(b), "toggled",
+		(GtkSignalFunc)hanja_subst_mode_toggle_cb,
 		GINT_TO_POINTER(i+1));
     }
     g_strfreev(hanja_formats);
+    g_string_free(fmts_xlat, TRUE);
 
     gtk_widget_show_all(win);
     gtk_widget_grab_focus(ctable);
@@ -219,7 +229,7 @@ hanja_dialog_selected(GtkWidget *widget, char *selected)
 	if (ic->han == NULL) {
 	    g_warning("%s: Hangul Buffer attached to special chars dialog is null",__FUNCTION__);
 	} else {
-	    if (ic->han == gtk_object_get_data((GtkObject *)top, "han")) 
+	    if (ic->han == gtk_object_get_data((GtkObject *)top, "han"))
 		editing_hanja_replace(ic, selected);
 	    else g_warning("%s: han is different from original.",__FUNCTION__);
 	}
@@ -259,9 +269,10 @@ hanja_dict_load(void)
 	fd = open(hanja_dic_filename, O_RDONLY);
     }
     if (fd < 0) {
-	g_warning("fail to open hanja dictionary %s", hanja_dic_filename);
-	g_print("hanja.dic Ͻõ ƴϸ hanja-hwp2ami ̿ؼ\n"
-		"hwp ڻ ƹ̿ ȯϿ Ͻʽÿ\n");
+	g_warning(_("Failed to open hanja dictionary %s."), hanja_dic_filename);
+	g_print(_("To enter Hanja, you need to install Hanja dictionary %s.\n"
+		  "Alternatively, you can convert a Hanja dictionary for HWP "
+		  "to a\n format understood by Ami."), hanja_dic_filename);
 	return ;
     }
     fstat(fd, &statbuf);
@@ -288,39 +299,71 @@ hanja_dict_close(void)
 }
 
 
-/*  Լ  gtkutilִ gtkdic  ͵Դϴ.*/
+/* The following functions are taken from gtkdic in gtkutils suit by NAM Sung-hyun */
 
 
 static char *
 go_next_line(char *s, char *last)
 {
-   while (s < last && *s != '\n')
-      s++;
-   if (s == last)
-      return NULL;
-   return s + 1;
+    while (s < last && *s != '\n')
+	s++;
+    if (s == last)
+	return NULL;
+    return s + 1;
 }
 
-/* sͽϴ  ó ̵Ѵ.
- * start   ó̹Ƿ lower bound ȴ.
+/* Move to the beg. of a line that start with s. 
+ * start is the beg. of this buffer and is the lower bound.
  */
 static char *
 go_start_of_line(char *s, char *start)
 {
-   if (s == start)
-      return start;
-   while (s > start && *s != '\n')
-      s--;
-   if (*s == '\n')
-      return s + 1;
-   return start;
+    if (s == start)
+	return start;
+    while (s > start && *s != '\n')
+	s--;
+    if (*s == '\n')
+	return s + 1;
+    return start;
 }
 
 static char *
-hanja_find_word(char *search)
+hanja_find_word(char *word)
 {
     char *e, *b, *s;
     int len, comp;
+    static iconv_t icd_to_utf8=NULL;
+    static iconv_t icd_to_euc=NULL;
+    char *search;
+    char *match;
+    char *last;
+
+    /* FIXME : eventually, we have to go the opposite direction.
+     * that is, Hanja dict. should be in UTF-8 and in EUC-KR
+     * locale, a search string has to be converted to UTF-8 before
+     * dict. look-up. J. Shin */
+
+    if ( ami_codeset==AMI_UTF8) {
+        /* U+AC00 -> EAB080, U+D7A3 -> ED9EA3 */
+	size_t ilen,olen;
+	char *iptr=word;
+	char *optr;
+       
+	ilen=olen=strlen(word)+1;
+     	if (! IS_UTF8_3BYTE(word[0]) || (unsigned char)word[0] < 0xea
+	      || (unsigned char)word[0] > 0xed )
+	    return NULL;
+    	if ( ! icd_to_euc || icd_to_euc == (iconv_t) -1 ) 
+	    icd_to_euc=iconv_open(EUCKR_ICD,"UTF-8");
+	search = g_new(gchar, strlen(word)+1);
+	optr=search;
+	if ( iconv(icd_to_euc,&iptr,&ilen,&optr,&olen) == -1) 
+	    return NULL;
+	if (ilen) *optr='\0';
+    }
+    else 
+	search = word;
+
     if ((unsigned char)search[0] < 0xb0) return NULL;
     if ((unsigned char)search[0] > 0xc8) return NULL;
 
@@ -329,26 +372,49 @@ hanja_find_word(char *search)
     b = hanjadic->buf;
     if (strncmp(b, search, len) == 0) return b;
 
-   /* bi-section ġϴ  ִ  ãƳ. */
-   while (1)
-   {
+    /* bi-section-search for a line with a given pattern */
+    while (1)
+    {
 
-      s = b + (e - b) / 2;
-      if ((s = go_start_of_line(s, b)) == NULL)
-         return NULL;
-      if (s == b)
-      {
-	 s = go_next_line(s, e);
-	 if (s == NULL || s == e)
-            return NULL;
-      }
-      comp = strncmp(s, search, len);
-      if (comp == 0)
-	 break;
-      else if (comp > 0)
-	 e = s;
-      else
-	 b = s;
-   }
-   return s;
+	s = b + (e - b) / 2;
+	if ((s = go_start_of_line(s, b)) == NULL)
+	    return NULL;
+	if (s == b)
+	{
+	    s = go_next_line(s, e);
+	    if (s == NULL || s == e)
+		return NULL;
+	}
+	comp = strncmp(s, search, len);
+	if (comp == 0)
+	    break;
+	else if (comp > 0)
+	    e = s;
+	else
+	    b = s;
+    }
+
+    last=strchr(s,'\n');
+    if ( last!=NULL ) *last='\0';
+    if ( ami_codeset==AMI_EUC) 
+	match=g_strdup(s);
+    else {
+	size_t ilen=strlen(s)+1;
+	size_t olen=ilen*2;
+	char *iptr=s;
+	char *optr;
+	match=g_new(gchar,olen);
+	optr=match;
+    	if ( ! icd_to_utf8 || icd_to_utf8 == (iconv_t) -1 ) 
+	    icd_to_utf8=iconv_open("UTF-8",EUCKR_ICD);
+	if ( iconv(icd_to_utf8,&iptr,&ilen,&optr,&olen) == -1) {
+	     g_warning("Something wrong with your dictionary.\n"
+	         "Conversion from EUC-KR to UTF-8 failed.\n");
+	     g_free(match);
+	     match=NULL;
+	}
+	if (ilen) *optr='\0'; 
+    }
+    if ( last!=NULL ) *last='\n';
+    return match;
 }
