@@ -41,7 +41,9 @@ ic_han_new(void)
 {
     HangulState *han;
     han = g_new0(HangulState, 1);
-    ami_hangul_state_init(han);
+    han->composer = composer_new();
+    if (ami_codeset == AMI_UTF8) 
+      composer_set_output_encoding(han->composer, "UTF-8");
     han->buf = g_new0(char, 256);
     han->buf_size = 256;
     return han;
@@ -51,6 +53,7 @@ void
 ic_han_free(HangulState *han)
 {
     if (han == NULL) return;
+    composer_free(han->composer);
     g_free(han->buf);
     g_free(han);
 }
@@ -218,7 +221,23 @@ ic_store(IC *ic, IMChangeICStruct *call_data)
 	//g_print("%s: ic_attr = %s; request\n", __FUNCTION__,ic_attr->name);
 	if (!strcmp(XNInputStyle, ic_attr->name)) {
 	  ic->input_style = *(INT32*)ic_attr->value;
-	  if(debug) g_print("input_style: %#lx of ic(%d)\n", ic->input_style, ic->id);
+	  if(debug) {
+	    g_print("input_style: id = %d\n", ic->id);
+
+#define pr_flag(a) if (ic->input_style & a) g_print( #a " ");
+
+	    pr_flag(XIMPreeditArea)
+	    pr_flag(XIMPreeditPosition)
+	    pr_flag(XIMPreeditCallbacks)
+	    pr_flag(XIMPreeditNothing)
+	    pr_flag(XIMPreeditNone)
+
+	    pr_flag(XIMStatusCallbacks)
+	    pr_flag(XIMStatusArea)
+	    pr_flag(XIMStatusNothing)
+	    pr_flag(XIMStatusNone)
+	    g_print("\n");
+	  }
 	  if (ic->input_style & (XIMPreeditArea|XIMPreeditPosition)) {
 	     ic->edit_win = dw_new(ic);
 	  }
@@ -233,7 +252,7 @@ ic_store(IC *ic, IMChangeICStruct *call_data)
     for (i = 0; i < (int)call_data->ic_attr_num; i++, ic_attr++) {
 	//g_print("%s: ic_attr = %s; request\n", __FUNCTION__,ic_attr->name);
 	if (!strcmp(XNClientWindow, ic_attr->name)) {
-	  Window win = *(Window*)ic_attr->value;
+	  Window win = (Window)*(INT32*)ic_attr->value;
 	  if (debug) g_print("client win set: xwin=%#lx ic = %d\n", win, ic->id);
 	  if (ic->client_win) {
 	    g_warning("client win is specified repeatedly");
@@ -263,7 +282,7 @@ ic_store(IC *ic, IMChangeICStruct *call_data)
 	      continue;
 	  }
 
-	  win = *(Window*)ic_attr->value;
+	  win = (Window)*(INT32*)ic_attr->value;
 	  fwin = gdk_window_lookup(win);
 
 	  if (fwin && (/*fwin == ic->client_win ||*/ fwin == ic->focus_win)) {
