@@ -3,7 +3,7 @@
 #include "config.h"
 
 /*
-Ʒ ƾ Ϻδ kinput2 ִ parsekey.cκ Խϴ.
+Some of routines below come from parsekey.c of kinput2.
 */
 
 /*- 
