@@ -4,20 +4,24 @@
 #include "config.h"
 
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include "amitype.h"
 #include "edit.h"
-#include <gdk/gdkkeysyms.h>
-
+#include "ami.h"
 #include "chartable.h"
+#include "utf8.h"
 
 
 static gint current_x = 0;
 static gint current_y = 0;
 static gint current_page_num = 0;
 
-static void char_frame(GtkWidget * notebook, char *title, char first, char second, int row, int col);
+static iconv_t icd_to_utf8=NULL;
 
-static void 
+static void char_frame(GtkWidget * notebook, char *title, char first,
+		       char second, int row, int col);
+
+static void
 si_destroyed(GtkWidget *win)
 {
     IC *ic;
@@ -37,8 +41,8 @@ key_press_cb(GtkWidget *w, GdkEventKey *ev)
 	GtkNotebook *notebook = gtk_object_get_data((GtkObject *)w, "notebook");
 	GtkWidget *ctable;
 	if (current_page_num == 7) /* last page; */
-	    gtk_notebook_set_page(notebook, 0); 
-	else 
+	    gtk_notebook_set_page(notebook, 0);
+	else
 	    gtk_notebook_next_page(notebook);
 	ctable = GTK_BIN(gtk_notebook_get_nth_page (notebook,gtk_notebook_get_current_page(notebook)))->child;
 	gtk_widget_grab_focus(ctable);
@@ -63,7 +67,7 @@ notebook_switch(GtkWidget *w, GtkNotebookPage *page, int page_num)
     gtk_widget_grab_focus((GtkWidget *)table);
 }
 
-GtkWidget * 
+GtkWidget *
 special_char_input(void)
 {
     GtkWidget *notebook;
@@ -76,7 +80,7 @@ special_char_input(void)
     gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER);
     gtk_signal_connect(GTK_OBJECT(win), "destroy",
 		       GTK_SIGNAL_FUNC(si_destroyed), NULL);
-    gtk_window_set_title(GTK_WINDOW(win), "ƹ: Ư Է");
+    gtk_window_set_title(GTK_WINDOW(win), _("Ami: Special/Symbol Character Input"));
     gtk_container_set_border_width(GTK_CONTAINER(win), 10);
     gtk_signal_connect((GtkObject *)win, "key_press_event", (GtkSignalFunc)key_press_cb, NULL);
     vbox = gtk_vbox_new(FALSE, 5);
@@ -90,17 +94,24 @@ special_char_input(void)
     gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
     gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
 
-    char_frame(notebook, "׸1", 0xa1, 0xa0, 6, 16);
-    char_frame(notebook, "׸2", 0xa2, 0xa0, 6, 12);
-    char_frame(notebook, "", 0xa3, 0xa0, 6, 16);
-    char_frame(notebook, "ѱڸ", 0xa4, 0xa0, 6, 16);
-    char_frame(notebook, "", 0xa6, 0xa0, 5, 14);
-    char_frame(notebook, "", 0xa7, 0xa0, 5, 16);
-    char_frame(notebook, "", 0xa8, 0xa0, 6, 16);
-    char_frame(notebook, "ȣ", 0xa9, 0xa0, 6, 16);
-    char_frame(notebook, "Ϻ1", 0xaa, 0xa0, 6, 16);
-    char_frame(notebook, "Ϻ2", 0xab, 0xa0, 6, 16);
-    char_frame(notebook, "þƾ", 0xac, 0xa0, 6, 16);
+    /* FIXME : codeset names are not standardized and OS-dependent. */
+    if (ami_codeset==AMI_UTF8 && (! icd_to_utf8 || icd_to_utf8 == (iconv_t) -1)) {
+        icd_to_utf8 = iconv_open("UTF-8", EUCKR_ICD);
+        g_assert(icd_to_utf8 != (iconv_t) (-1));
+    }
+
+    char_frame(notebook, _("Math etc"), 0xa1, 0xa0, 6, 16);
+    char_frame(notebook, _("Var. symbols"), 0xa2, 0xa0, 6, 12);
+    char_frame(notebook, _("Fullwidth ASCII"), 0xa3, 0xa0, 6, 16);
+    char_frame(notebook, _("Hangul Jamo"), 0xa4, 0xa0, 6, 16);
+    char_frame(notebook, _("Greek"), 0xa5, 0xa0, 6, 16);
+    char_frame(notebook, _("Box drawings"), 0xa6, 0xa0, 5, 14);
+    char_frame(notebook, _("Meas. units"), 0xa7, 0xa0, 5, 16);
+    char_frame(notebook, _("Circled"), 0xa8, 0xa0, 6, 16);
+    char_frame(notebook, _("Parenthesized"), 0xa9, 0xa0, 6, 16);
+    char_frame(notebook, _("Hiragana"), 0xaa, 0xa0, 6, 16);
+    char_frame(notebook, _("Katakana"), 0xab, 0xa0, 6, 16);
+    char_frame(notebook, _("Cyrillic"), 0xac, 0xa0, 6, 16);
     x = current_x; y = current_y;
     num = current_page_num;
     gtk_notebook_set_page((GtkNotebook *)notebook, num);
@@ -124,7 +135,7 @@ char_selected(GtkWidget *widget, char *s)
 	if (ic->han == NULL) {
 	    g_warning("%s: Hangul Buffer attached to special chars dialog is null",__FUNCTION__);
 	} else {
-	    if (ic->han == gtk_object_get_data((GtkObject *)top, "han")) 
+	    if (ic->han == gtk_object_get_data((GtkObject *)top, "han"))
 		editing_insert(ic, s, strlen(s));
 	    else g_warning("%s: han is different from original.",__FUNCTION__);
 	}
@@ -139,23 +150,44 @@ char_moved(GtkWidget *w, gint x, gint y)
     current_y = y;
 }
 
-static void 
-char_frame(GtkWidget * notebook, char *title, 
+static void
+char_frame(GtkWidget * notebook, char *title,
            char first, char second, int row, int col)
 {
     GtkWidget *frame, *label;
     GtkWidget *table;
     int i, k;
     char **buf;
-
+    int len= ami_codeset==AMI_UTF8 ? 3 : 2 ;  /* Even in UTF-8, some special 
+						 chars take only two bytes, but
+						 we can be lazy here. */ 
     buf = g_new(char *, col*row);
     for (i = 0; i < col*row; i++)
-	buf[i] = g_new(char, 3);
+	buf[i] = g_new(char, len+1);
     k = second;
+    if ( ami_codeset==AMI_EUC) 
     for (i = 0; i < row*col; i++) {
 	buf[i][0] = first;
 	buf[i][1] = k++;
 	buf[i][2] = '\0';
+	}
+    else {
+        char tbuf[3];
+	char *iptr,*optr;
+	size_t ilen,olen;
+	for (i = 0; i < row*col; i++) {
+	    tbuf[0] = first;
+	    tbuf[1] = k++;
+	    tbuf[2] = '\0';
+	    iptr=tbuf;
+	    optr=buf[i];
+	    ilen=3;
+	    olen=4; 
+	    if ( iconv(icd_to_utf8,&iptr,&ilen,&optr,&olen) == (size_t) -1) 
+		strcpy(buf[i],"\xef\xbf\xbd");  /*U+FFFD in UTF-8 */
+	    else 
+	      if (olen) *optr='\0';
+	}
     }
 
     frame = gtk_frame_new(NULL);
@@ -164,9 +196,9 @@ char_frame(GtkWidget * notebook, char *title,
 
     table = char_table_new(buf, col, col*row);
     gtk_container_add(GTK_CONTAINER(frame), table);
-    gtk_signal_connect(GTK_OBJECT(table), "selected", 
+    gtk_signal_connect(GTK_OBJECT(table), "selected",
     	GTK_SIGNAL_FUNC(char_selected), NULL);
-    gtk_signal_connect(GTK_OBJECT(table), "moved", 
+    gtk_signal_connect(GTK_OBJECT(table), "moved",
     	GTK_SIGNAL_FUNC(char_moved), NULL);
     gtk_widget_show(table);
     label = gtk_label_new(title);
